# encoding: utf-8

"""
Enumerations used by text and related objects
"""

from __future__ import absolute_import

from .base import (
    alias,
    Enumeration,
    EnumMember,
    ReturnValueOnlyEnumMember,
    XmlEnumeration,
    XmlMappedEnumMember,
)


class MSO_AUTO_SIZE(Enumeration):
    """
    Determines the type of automatic sizing allowed.

    The following names can be used to specify the automatic sizing behavior
    used to fit a shape's text within the shape bounding box, for example::

        from pptx.enum.text import MSO_AUTO_SIZE

        shape.text_frame.auto_size = MSO_AUTO_SIZE.TEXT_TO_FIT_SHAPE

    The word-wrap setting of the text frame interacts with the auto-size
    setting to determine the specific auto-sizing behavior.

    Note that ``TextFrame.auto_size`` can also be set to |None|, which removes
    the auto size setting altogether. This causes the setting to be inherited,
    either from the layout placeholder, in the case of a placeholder shape, or
    from the theme.
    """

    NONE = 0
    SHAPE_TO_FIT_TEXT = 1
    TEXT_TO_FIT_SHAPE = 2

    __ms_name__ = "MsoAutoSize"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff865367(v=office.15" ").aspx"
    )

    __members__ = (
        EnumMember(
            "NONE",
            0,
            "No automatic sizing of the shape or text will be don"
            "e. Text can freely extend beyond the horizontal and vertical ed"
            "ges of the shape bounding box.",
        ),
        EnumMember(
            "SHAPE_TO_FIT_TEXT",
            1,
            "The shape height and possibly width are"
            " adjusted to fit the text. Note this setting interacts with the"
            " TextFrame.word_wrap property setting. If word wrap is turned o"
            "n, only the height of the shape will be adjusted; soft line bre"
            "aks will be used to fit the text horizontally.",
        ),
        EnumMember(
            "TEXT_TO_FIT_SHAPE",
            2,
            "The font size is reduced as necessary t"
            "o fit the text within the shape.",
        ),
        ReturnValueOnlyEnumMember(
            "MIXED",
            -2,
            "Return value only; indicates a combination of auto"
            "matic sizing schemes are used.",
        ),
    )


@alias("MSO_UNDERLINE")
class MSO_TEXT_UNDERLINE_TYPE(XmlEnumeration):
    """
    Indicates the type of underline for text. Used with
    :attr:`.Font.underline` to specify the style of text underlining.

    Alias: ``MSO_UNDERLINE``

    Example::

        from pptx.enum.text import MSO_UNDERLINE

        run.font.underline = MSO_UNDERLINE.DOUBLE_LINE
    """

    __ms_name__ = "MsoTextUnderlineType"

    __url__ = "http://msdn.microsoft.com/en-us/library/aa432699.aspx"

    __members__ = (
        XmlMappedEnumMember("NONE", 0, "none", "Specifies no underline."),
        XmlMappedEnumMember(
            "DASH_HEAVY_LINE", 8, "dashHeavy", "Specifies a dash underline."
        ),
        XmlMappedEnumMember("DASH_LINE", 7, "dash", "Specifies a dash line underline."),
        XmlMappedEnumMember(
            "DASH_LONG_HEAVY_LINE",
            10,
            "dashLongHeavy",
            "Specifies a long heavy line underline.",
        ),
        XmlMappedEnumMember(
            "DASH_LONG_LINE", 9, "dashLong", "Specifies a dashed long line underline."
        ),
        XmlMappedEnumMember(
            "DOT_DASH_HEAVY_LINE",
            12,
            "dotDashHeavy",
            "Specifies a dot dash heavy line underline.",
        ),
        XmlMappedEnumMember(
            "DOT_DASH_LINE", 11, "dotDash", "Specifies a dot dash line underline."
        ),
        XmlMappedEnumMember(
            "DOT_DOT_DASH_HEAVY_LINE",
            14,
            "dotDotDashHeavy",
            "Specifies a dot dot dash heavy line underline.",
        ),
        XmlMappedEnumMember(
            "DOT_DOT_DASH_LINE",
            13,
            "dotDotDash",
            "Specifies a dot dot dash line underline.",
        ),
        XmlMappedEnumMember(
            "DOTTED_HEAVY_LINE",
            6,
            "dottedHeavy",
            "Specifies a dotted heavy line underline.",
        ),
        XmlMappedEnumMember(
            "DOTTED_LINE", 5, "dotted", "Specifies a dotted line underline."
        ),
        XmlMappedEnumMember(
            "DOUBLE_LINE", 3, "dbl", "Specifies a double line underline."
        ),
        XmlMappedEnumMember(
            "HEAVY_LINE", 4, "heavy", "Specifies a heavy line underline."
        ),
        XmlMappedEnumMember(
            "SINGLE_LINE", 2, "sng", "Specifies a single line underline."
        ),
        XmlMappedEnumMember(
            "WAVY_DOUBLE_LINE", 17, "wavyDbl", "Specifies a wavy double line underline."
        ),
        XmlMappedEnumMember(
            "WAVY_HEAVY_LINE", 16, "wavyHeavy", "Specifies a wavy heavy line underline."
        ),
        XmlMappedEnumMember(
            "WAVY_LINE", 15, "wavy", "Specifies a wavy line underline."
        ),
        XmlMappedEnumMember("WORDS", 1, "words", "Specifies underlining words."),
        ReturnValueOnlyEnumMember("MIXED", -2, "Specifies a mixed of underline types."),
    )


@alias("MSO_ANCHOR")
class MSO_VERTICAL_ANCHOR(XmlEnumeration):
    """
    Specifies the vertical alignment of text in a text frame. Used with the
    ``.vertical_anchor`` property of the |TextFrame| object. Note that the
    ``vertical_anchor`` property can also have the value None, indicating
    there is no directly specified vertical anchor setting and its effective
    value is inherited from its placeholder if it has one or from the theme.
    |None| may also be assigned to remove an explicitly specified vertical
    anchor setting.
    """

    __ms_name__ = "MsoVerticalAnchor"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff865255.aspx"

    __members__ = (
        XmlMappedEnumMember(
            None,
            None,
            None,
            "Text frame has no vertical anchor specified "
            "and inherits its value from its layout placeholder or theme.",
        ),
        XmlMappedEnumMember("TOP", 1, "t", "Aligns text to top of text frame"),
        XmlMappedEnumMember("MIDDLE", 3, "ctr", "Centers text vertically"),
        XmlMappedEnumMember("BOTTOM", 4, "b", "Aligns text to bottom of text frame"),
        ReturnValueOnlyEnumMember(
            "MIXED",
            -2,
            "Return value only; indicates a combination of the " "other states.",
        ),
    )


@alias("PP_ALIGN")
class PP_PARAGRAPH_ALIGNMENT(XmlEnumeration):
    """
    Specifies the horizontal alignment for one or more paragraphs.

    Alias: ``PP_ALIGN``

    Example::

        from pptx.enum.text import PP_ALIGN

        shape.paragraphs[0].alignment = PP_ALIGN.CENTER
    """

    __ms_name__ = "PpParagraphAlignment"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff745375(v=office.15" ").aspx"
    )

    __members__ = (
        XmlMappedEnumMember("CENTER", 2, "ctr", "Center align"),
        XmlMappedEnumMember(
            "DISTRIBUTE",
            5,
            "dist",
            "Evenly distributes e.g. Japanese chara"
            "cters from left to right within a line",
        ),
        XmlMappedEnumMember(
            "JUSTIFY",
            4,
            "just",
            "Justified, i.e. each line both begins and"
            " ends at the margin with spacing between words adjusted such th"
            "at the line exactly fills the width of the paragraph.",
        ),
        XmlMappedEnumMember(
            "JUSTIFY_LOW",
            7,
            "justLow",
            "Justify using a small amount of sp" "ace between words.",
        ),
        XmlMappedEnumMember("LEFT", 1, "l", "Left aligned"),
        XmlMappedEnumMember("RIGHT", 3, "r", "Right aligned"),
        XmlMappedEnumMember("THAI_DISTRIBUTE", 6, "thaiDist", "Thai distributed"),
        ReturnValueOnlyEnumMember(
            "MIXED",
            -2,
            "Return value only; indicates multiple paragraph al"
            "ignments are present in a set of paragraphs.",
        ),
    )



# encoding: utf-8

"""
Gherkin step implementations for chart features.
"""

from __future__ import absolute_import, print_function

import hashlib

from itertools import islice

from behave import given, then, when

from pptx import Presentation
from pptx.chart.chart import Legend
from pptx.chart.data import BubbleChartData, CategoryChartData, ChartData, XyChartData
from pptx.enum.chart import XL_CHART_TYPE
from pptx.parts.embeddedpackage import EmbeddedXlsxPart
from pptx.util import Inches

from helpers import count, test_pptx


# given ===================================================


@given("a Chart object as chart")
def given_a_Chart_object_as_chart(context):
    slide = Presentation(test_pptx("shp-common-props")).slides[0]
    context.chart = slide.shapes[6].chart


@given("a chart having {a_or_no} title")
def given_a_chart_having_a_or_no_title(context, a_or_no):
    shape_idx = {"no": 0, "a": 1}[a_or_no]
    prs = Presentation(test_pptx("cht-chart-props"))
    context.chart = prs.slides[0].shapes[shape_idx].chart


@given("a chart {having_or_not} a legend")
def given_a_chart_having_or_not_a_legend(context, having_or_not):
    slide_idx = {"having": 0, "not having": 1}[having_or_not]
    prs = Presentation(test_pptx("cht-legend"))
    context.chart = prs.slides[slide_idx].shapes[0].chart


@given("a chart of size and type {spec}")
def given_a_chart_of_size_and_type_spec(context, spec):
    slide_idx = {
        "2x2 Clustered Bar": 0,
        "2x2 100% Stacked Bar": 1,
        "2x2 Clustered Column": 2,
        "4x3 Line": 3,
        "3x1 Pie": 4,
        "3x2 XY": 5,
        "3x2 Bubble": 6,
    }[spec]
    prs = Presentation(test_pptx("cht-replace-data"))
    chart = prs.slides[slide_idx].shapes[0].chart
    context.chart = chart
    context.xlsx_sha1 = hashlib.sha1(chart._workbook.xlsx_part.blob).hexdigest()


@given("a chart of type {chart_type}")
def given_a_chart_of_type_chart_type(context, chart_type):
    slide_idx, shape_idx = {
        "Area": (0, 0),
        "Stacked Area": (0, 1),
        "100% Stacked Area": (0, 2),
        "3-D Area": (0, 3),
        "3-D Stacked Area": (0, 4),
        "3-D 100% Stacked Area": (0, 5),
        "Clustered Bar": (1, 0),
        "Stacked Bar": (1, 1),
        "100% Stacked Bar": (1, 2),
        "Clustered Column": (1, 3),
        "Stacked Column": (1, 4),
        "100% Stacked Column": (1, 5),
        "Line": (2, 0),
        "Stacked Line": (2, 1),
        "100% Stacked Line": (2, 2),
        "Marked Line": (2, 3),
        "Stacked Marked Line": (2, 4),
        "100% Stacked Marked Line": (2, 5),
        "Pie": (3, 0),
        "Exploded Pie": (3, 1),
        "XY (Scatter)": (4, 0),
        "XY Lines": (4, 1),
        "XY Lines No Markers": (4, 2),
        "XY Smooth Lines": (4, 3),
        "XY Smooth No Markers": (4, 4),
        "Bubble": (5, 0),
        "3D-Bubble": (5, 1),
        "Radar": (6, 0),
        "Marked Radar": (6, 1),
        "Filled Radar": (6, 2),
        "Line (with date categories)": (7, 0),
    }[chart_type]
    prs = Presentation(test_pptx("cht-chart-type"))
    context.chart = prs.slides[slide_idx].shapes[shape_idx].chart


@given("a chart title")
def given_a_chart_title(context):
    prs = Presentation(test_pptx("cht-chart-props"))
    context.chart_title = prs.slides[0].shapes[1].chart.chart_title


@given("a chart title having {a_or_no} text frame")
def given_a_chart_title_having_a_or_no_text_frame(context, a_or_no):
    prs = Presentation(test_pptx("cht-chart-props"))
    shape_idx = {"no": 0, "a": 1}[a_or_no]
    context.chart_title = prs.slides[1].shapes[shape_idx].chart.chart_title


# when ====================================================


@when("I add a Clustered bar chart with multi-level categories")
def when_I_add_a_clustered_bar_chart_with_multi_level_categories(context):
    chart_type = XL_CHART_TYPE.BAR_CLUSTERED
    chart_data = CategoryChartData()

    WEST = chart_data.add_category("WEST")
    WEST.add_sub_category("SF")
    WEST.add_sub_category("LA")
    EAST = chart_data.add_category("EAST")
    EAST.add_sub_category("NY")
    EAST.add_sub_category("NJ")

    chart_data.add_series("Series 1", (1, 2, None, 4))
    chart_data.add_series("Series 2", (5, None, 7, 8))

    context.chart = context.slide.shapes.add_chart(
        chart_type, Inches(1), Inches(1), Inches(8), Inches(5), chart_data
    ).chart


@when("I add a {kind} chart with {cats} categories and {sers} series")
def when_I_add_a_chart_with_categories_and_series(context, kind, cats, sers):
    chart_type = {
        "Area": XL_CHART_TYPE.AREA,
        "Stacked Area": XL_CHART_TYPE.AREA_STACKED,
        "100% Stacked Area": XL_CHART_TYPE.AREA_STACKED_100,
        "Clustered Bar": XL_CHART_TYPE.BAR_CLUSTERED,
        "Stacked Bar": XL_CHART_TYPE.BAR_STACKED,
        "100% Stacked Bar": XL_CHART_TYPE.BAR_STACKED_100,
        "Clustered Column": XL_CHART_TYPE.COLUMN_CLUSTERED,
        "Stacked Column": XL_CHART_TYPE.COLUMN_STACKED,
        "100% Stacked Column": XL_CHART_TYPE.COLUMN_STACKED_100,
        "Doughnut": XL_CHART_TYPE.DOUGHNUT,
        "Exploded Doughnut": XL_CHART_TYPE.DOUGHNUT_EXPLODED,
        "Line": XL_CHART_TYPE.LINE,
        "Line with Markers": XL_CHART_TYPE.LINE_MARKERS,
        "Line Markers Stacked": XL_CHART_TYPE.LINE_MARKERS_STACKED,
        "100% Line Markers Stacked": XL_CHART_TYPE.LINE_MARKERS_STACKED_100,
        "Line Stacked": XL_CHART_TYPE.LINE_STACKED,
        "100% Line Stacked": XL_CHART_TYPE.LINE_STACKED_100,
        "Pie": XL_CHART_TYPE.PIE,
        "Exploded Pie": XL_CHART_TYPE.PIE_EXPLODED,
        "Radar": XL_CHART_TYPE.RADAR,
        "Filled Radar": XL_CHART_TYPE.RADAR_FILLED,
        "Radar with markers": XL_CHART_TYPE.RADAR_MARKERS,
    }[kind]
    category_count, series_count = int(cats), int(sers)
    category_source = ("Foo", "Bar", "Baz", "Boo", "Far", "Faz")
    series_value_source = count(1.1, 1.1)

    chart_data = CategoryChartData()
    chart_data.categories = category_source[:category_count]
    for idx in range(series_count):
        series_title = "Series %d" % (idx + 1)
        series_values = tuple(islice(series_value_source, category_count))
        chart_data.add_series(series_title, series_values)

    context.chart = context.slide.shapes.add_chart(
        chart_type, Inches(1), Inches(1), Inches(8), Inches(5), chart_data
    ).chart


@when("I add a {bubble_type} chart having 2 series of 3 points each")
def when_I_add_a_bubble_chart_having_2_series_of_3_pts(context, bubble_type):
    chart_type = getattr(XL_CHART_TYPE, bubble_type)
    data = (
        ("Series 1", ((-0.1, 0.5, 1.0), (16.2, 0.0, 2.0), (8.0, -0.2, 3.0))),
        ("Series 2", ((12.4, 0.8, 4.0), (-7.5, 0.5, 5.0), (5.1, -0.5, 6.0))),
    )

    chart_data = BubbleChartData()

    for series_data in data:
        series_label, points = series_data
        series = chart_data.add_series(series_label)
        for point in points:
            x, y, size = point
            series.add_data_point(x, y, size)

    context.chart = context.slide.shapes.add_chart(
        chart_type, Inches(1), Inches(1), Inches(8), Inches(5), chart_data
    ).chart


@when("I assign {value} to chart.has_legend")
def when_I_assign_value_to_chart_has_legend(context, value):
    new_value = {"True": True, "False": False}[value]
    context.chart.has_legend = new_value


@when("I assign {value} to chart.has_title")
def when_I_assign_value_to_chart_has_title(context, value):
    context.chart.has_title = {"True": True, "False": False}[value]


@when("I assign {value} to chart_title.has_text_frame")
def when_I_assign_value_to_chart_title_has_text_frame(context, value):
    context.chart_title.has_text_frame = {"True": True, "False": False}[value]


@when("I replace its data with {cats} categories and {sers} series")
def when_I_replace_its_data_with_categories_and_series(context, cats, sers):
    category_count, series_count = int(cats), int(sers)
    category_source = ("Foo", "Bar", "Baz", "Boo", "Far", "Faz")
    series_value_source = count(1.1, 1.1)

    chart_data = ChartData()
    chart_data.categories = category_source[:category_count]
    for idx in range(series_count):
        series_title = "New Series %d" % (idx + 1)
        series_values = tuple(islice(series_value_source, category_count))
        chart_data.add_series(series_title, series_values)

    context.chart.replace_data(chart_data)


@when("I replace its data with 3 series of 3 bubble points each")
def when_I_replace_its_data_with_3_series_of_three_bubble_pts_each(context):
    chart_data = BubbleChartData()
    for idx in range(3):
        series_title = "New Series %d" % (idx + 1)
        series = chart_data.add_series(series_title)
        for jdx in range(3):
            x, y, size = idx * 3 + jdx, idx * 2 + jdx, idx + jdx
            series.add_data_point(x, y, size)

    context.chart.replace_data(chart_data)


@when("I replace its data with 3 series of 3 points each")
def when_I_replace_its_data_with_3_series_of_three_points_each(context):
    chart_data = XyChartData()
    x = y = 0
    for idx in range(3):
        series_title = "New Series %d" % (idx + 1)
        series = chart_data.add_series(series_title)
        for jdx in range(3):
            x, y = idx * 3 + jdx, idx * 2 + jdx
            series.add_data_point(x, y)

    context.chart.replace_data(chart_data)


# then ====================================================


@then("chart.category_axis is a {cls_name} object")
def then_chart_category_axis_is_a_cls_name_object(context, cls_name):
    category_axis = context.chart.category_axis
    type_name = type(category_axis).__name__
    assert type_name == cls_name, "got %s" % type_name


@then("chart.chart_title is a ChartTitle object")
def then_chart_chart_title_is_a_ChartTitle_object(context):
    class_name = type(context.chart.chart_title).__name__
    assert class_name == "ChartTitle", "got %s" % class_name


@then("chart.chart_type is {enum_member}")
def then_chart_chart_type_is_value(context, enum_member):
    expected_value = getattr(XL_CHART_TYPE, enum_member)
    chart = context.chart
    assert chart.chart_type is expected_value, "got %s" % chart.chart_type


@then("chart.font is a Font object")
def then_chart_font_is_a_Font_object(context):
    actual = type(context.chart.font).__name__
    expected = "Font"
    assert actual == expected, "chart.font is a %s object" % actual


@then("chart.has_legend is {value}")
def then_chart_has_legend_is_value(context, value):
    expected_value = {"True": True, "False": False}[value]
    chart = context.chart
    assert chart.has_legend is expected_value


@then("chart.has_title is {value}")
def then_chart_has_title_is_value(context, value):
    chart = context.chart
    actual_value = chart.has_title
    expected_value = {"True": True, "False": False}[value]
    assert actual_value is expected_value, "got %s" % actual_value


@then("chart.legend is a legend object")
def then_chart_legend_is_a_legend_object(context):
    chart = context.chart
    assert isinstance(chart.legend, Legend)


@then("chart.series is a SeriesCollection object")
def then_chart_series_is_a_SeriesCollection_object(context):
    type_name = type(context.chart.series).__name__
    assert type_name == "SeriesCollection", "got %s" % type_name


@then("chart.value_axis is a ValueAxis object")
def then_chart_value_axis_is_a_ValueAxis_object(context):
    value_axis = context.chart.value_axis
    assert type(value_axis).__name__ == "ValueAxis"


@then("chart_title.format is a ChartFormat object")
def then_chart_title_format_is_a_ChartFormat_object(context):
    class_name = type(context.chart_title.format).__name__
    assert class_name == "ChartFormat", "got %s" % class_name


@then("chart_title.format.fill is a FillFormat object")
def then_chart_title_format_fill_is_a_FillFormat_object(context):
    class_name = type(context.chart_title.format.fill).__name__
    assert class_name == "FillFormat", "got %s" % class_name


@then("chart_title.format.line is a LineFormat object")
def then_chart_title_format_line_is_a_LineFormat_object(context):
    class_name = type(context.chart_title.format.line).__name__
    assert class_name == "LineFormat", "got %s" % class_name


@then("chart_title.has_text_frame is {value}")
def then_chart_title_has_text_frame_is_value(context, value):
    actual_value = context.chart_title.has_text_frame
    expected_value = {"True": True, "False": False}[value]
    assert actual_value is expected_value, "got %s" % actual_value


@then("chart_title.text_frame is a TextFrame object")
def then_chart_title_text_frame_is_a_TextFrame_object(context):
    class_name = type(context.chart_title.text_frame).__name__
    assert class_name == "TextFrame", "got %s" % class_name


@then("each series has a new name")
def then_each_series_has_a_new_name(context):
    for series in context.chart.plots[0].series:
        assert series.name.startswith("New ")


@then("each series has {count} values")
def then_each_series_has_count_values(context, count):
    expected_count = int(count)
    for series in context.chart.plots[0].series:
        actual_value_count = len(series.values)
        assert actual_value_count == expected_count


@then("len(chart.series) is {count}")
def then_len_chart_series_is_count(context, count):
    expected_count = int(count)
    assert len(context.chart.series) == expected_count


@then("the chart has an Excel data worksheet")
def then_the_chart_has_an_Excel_data_worksheet(context):
    xlsx_part = context.chart._workbook.xlsx_part
    assert isinstance(xlsx_part, EmbeddedXlsxPart)


@then("the chart has new chart data")
def then_the_chart_has_new_chart_data(context):
    orig_xlsx_sha1 = context.xlsx_sha1
    new_xlsx_sha1 = hashlib.sha1(context.chart._workbook.xlsx_part.blob).hexdigest()
    assert new_xlsx_sha1 != orig_xlsx_sha1



# encoding: utf-8

"""Base shape-related objects such as BaseShape."""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.action import ActionSetting
from pptx.dml.effect import ShadowFormat
from pptx.shared import ElementProxy
from pptx.util import lazyproperty


class BaseShape(object):
    """Base class for shape objects.

    Subclasses include |Shape|, |Picture|, and |GraphicFrame|.
    """

    def __init__(self, shape_elm, parent):
        super(BaseShape, self).__init__()
        self._element = shape_elm
        self._parent = parent

    def __eq__(self, other):
        """|True| if this shape object proxies the same element as *other*.

        Equality for proxy objects is defined as referring to the same XML
        element, whether or not they are the same proxy object instance.
        """
        if not isinstance(other, BaseShape):
            return False
        return self._element is other._element

    def __ne__(self, other):
        if not isinstance(other, BaseShape):
            return True
        return self._element is not other._element

    @lazyproperty
    def click_action(self):
        """|ActionSetting| instance providing access to click behaviors.

        Click behaviors are hyperlink-like behaviors including jumping to
        a hyperlink (web page) or to another slide in the presentation. The
        click action is that defined on the overall shape, not a run of text
        within the shape. An |ActionSetting| object is always returned, even
        when no click behavior is defined on the shape.
        """
        cNvPr = self._element._nvXxPr.cNvPr
        return ActionSetting(cNvPr, self)

    @property
    def element(self):
        """`lxml` element for this shape, e.g. a CT_Shape instance.

        Note that manipulating this element improperly can produce an invalid
        presentation file. Make sure you know what you're doing if you use
        this to change the underlying XML.
        """
        return self._element

    @property
    def has_chart(self):
        """
        |True| if this shape is a graphic frame containing a chart object.
        |False| otherwise. When |True|, the chart object can be accessed
        using the ``.chart`` property.
        """
        # This implementation is unconditionally False, the True version is
        # on GraphicFrame subclass.
        return False

    @property
    def has_table(self):
        """
        |True| if this shape is a graphic frame containing a table object.
        |False| otherwise. When |True|, the table object can be accessed
        using the ``.table`` property.
        """
        # This implementation is unconditionally False, the True version is
        # on GraphicFrame subclass.
        return False

    @property
    def has_text_frame(self):
        """
        |True| if this shape can contain text.
        """
        # overridden on Shape to return True. Only <p:sp> has text frame
        return False

    @property
    def height(self):
        """
        Read/write. Integer distance between top and bottom extents of shape
        in EMUs
        """
        return self._element.cy

    @height.setter
    def height(self, value):
        self._element.cy = value

    @property
    def is_placeholder(self):
        """
        True if this shape is a placeholder. A shape is a placeholder if it
        has a <p:ph> element.
        """
        return self._element.has_ph_elm

    @property
    def left(self):
        """
        Read/write. Integer distance of the left edge of this shape from the
        left edge of the slide, in English Metric Units (EMU)
        """
        return self._element.x

    @left.setter
    def left(self, value):
        self._element.x = value

    @property
    def name(self):
        """
        Name of this shape, e.g. 'Picture 7'
        """
        return self._element.shape_name

    @name.setter
    def name(self, value):
        self._element._nvXxPr.cNvPr.name = value

    @property
    def part(self):
        """The package part containing this shape.

        A |BaseSlidePart| subclass in this case. Access to a slide part
        should only be required if you are extending the behavior of |pp| API
        objects.
        """
        return self._parent.part

    @property
    def placeholder_format(self):
        """
        A |_PlaceholderFormat| object providing access to
        placeholder-specific properties such as placeholder type. Raises
        |ValueError| on access if the shape is not a placeholder.
        """
        if not self.is_placeholder:
            raise ValueError("shape is not a placeholder")
        return _PlaceholderFormat(self._element.ph)

    @property
    def rotation(self):
        """
        Read/write float. Degrees of clockwise rotation. Negative values can
        be assigned to indicate counter-clockwise rotation, e.g. assigning
        -45.0 will change setting to 315.0.
        """
        return self._element.rot

    @rotation.setter
    def rotation(self, value):
        self._element.rot = value

    @lazyproperty
    def shadow(self):
        """|ShadowFormat| object providing access to shadow for this shape.

        A |ShadowFormat| object is always returned, even when no shadow is
        explicitly defined on this shape (i.e. it inherits its shadow
        behavior).
        """
        return ShadowFormat(self._element.spPr)

    @property
    def shape_id(self):
        """Read-only positive integer identifying this shape.

        The id of a shape is unique among all shapes on a slide.
        """
        return self._element.shape_id

    @property
    def shape_type(self):
        """
        Unique integer identifying the type of this shape, like
        ``MSO_SHAPE_TYPE.CHART``. Must be implemented by subclasses.
        """
        # # This one returns |None| unconditionally to account for shapes
        # # that haven't been implemented yet, like group shape and chart.
        # # Once those are done this should raise |NotImplementedError|.
        # msg = 'shape_type property must be implemented by subclasses'
        # raise NotImplementedError(msg)
        return None

    @property
    def top(self):
        """
        Read/write. Integer distance of the top edge of this shape from the
        top edge of the slide, in English Metric Units (EMU)
        """
        return self._element.y

    @top.setter
    def top(self, value):
        self._element.y = value

    @property
    def width(self):
        """
        Read/write. Integer distance between left and right extents of shape
        in EMUs
        """
        return self._element.cx

    @width.setter
    def width(self, value):
        self._element.cx = value


class _PlaceholderFormat(ElementProxy):
    """
    Accessed via the :attr:`~.BaseShape.placeholder_format` property of
    a placeholder shape, provides properties specific to placeholders, such
    as the placeholder type.
    """

    @property
    def element(self):
        """
        The `p:ph` element proxied by this object.
        """
        return super(_PlaceholderFormat, self).element

    @property
    def idx(self):
        """
        Integer placeholder 'idx' attribute.
        """
        return self._element.idx

    @property
    def type(self):
        """
        Placeholder type, a member of the :ref:`PpPlaceholderType`
        enumeration, e.g. PP_PLACEHOLDER.CHART
        """
        return self._element.type



# encoding: utf-8

"""Mappings from the ISO/IEC 29500 spec.

Some of these are inferred from PowerPoint application behavior
"""

from pptx.enum.shapes import MSO_SHAPE


GRAPHIC_DATA_URI_CHART = "http://schemas.openxmlformats.org/drawingml/2006/chart"
GRAPHIC_DATA_URI_OLEOBJ = "http://schemas.openxmlformats.org/presentationml/2006/ole"
GRAPHIC_DATA_URI_TABLE = "http://schemas.openxmlformats.org/drawingml/2006/table"


# ============================================================================
# AutoShape type specs
# ============================================================================

autoshape_types = {
    MSO_SHAPE.ACTION_BUTTON_BACK_OR_PREVIOUS: {
        "basename": "Action Button: Back or Previous",
        "avLst": (),
    },
    MSO_SHAPE.ACTION_BUTTON_BEGINNING: {
        "basename": "Action Button: Beginning",
        "avLst": (),
    },
    MSO_SHAPE.ACTION_BUTTON_CUSTOM: {"basename": "Action Button: Custom", "avLst": ()},
    MSO_SHAPE.ACTION_BUTTON_DOCUMENT: {
        "basename": "Action Button: Document",
        "avLst": (),
    },
    MSO_SHAPE.ACTION_BUTTON_END: {"basename": "Action Button: End", "avLst": ()},
    MSO_SHAPE.ACTION_BUTTON_FORWARD_OR_NEXT: {
        "basename": "Action Button: Forward or Next",
        "avLst": (),
    },
    MSO_SHAPE.ACTION_BUTTON_HELP: {"basename": "Action Button: Help", "avLst": ()},
    MSO_SHAPE.ACTION_BUTTON_HOME: {"basename": "Action Button: Home", "avLst": ()},
    MSO_SHAPE.ACTION_BUTTON_INFORMATION: {
        "basename": "Action Button: Information",
        "avLst": (),
    },
    MSO_SHAPE.ACTION_BUTTON_MOVIE: {"basename": "Action Button: Movie", "avLst": ()},
    MSO_SHAPE.ACTION_BUTTON_RETURN: {"basename": "Action Button: Return", "avLst": ()},
    MSO_SHAPE.ACTION_BUTTON_SOUND: {"basename": "Action Button: Sound", "avLst": ()},
    MSO_SHAPE.ARC: {"basename": "Arc", "avLst": (("adj1", 16200000), ("adj2", 0))},
    MSO_SHAPE.BALLOON: {
        "basename": "Rounded Rectangular Callout",
        "avLst": (("adj1", -20833), ("adj2", 62500), ("adj3", 16667)),
    },
    MSO_SHAPE.BENT_ARROW: {
        "basename": "Bent Arrow",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000), ("adj4", 43750)),
    },
    MSO_SHAPE.BENT_UP_ARROW: {
        "basename": "Bent-Up Arrow",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000)),
    },
    MSO_SHAPE.BEVEL: {"basename": "Bevel", "avLst": (("adj", 12500),)},
    MSO_SHAPE.BLOCK_ARC: {
        "basename": "Block Arc",
        "avLst": (("adj1", 10800000), ("adj2", 0), ("adj3", 25000)),
    },
    MSO_SHAPE.CAN: {"basename": "Can", "avLst": (("adj", 25000),)},
    MSO_SHAPE.CHART_PLUS: {"basename": "Chart Plus", "avLst": ()},
    MSO_SHAPE.CHART_STAR: {"basename": "Chart Star", "avLst": ()},
    MSO_SHAPE.CHART_X: {"basename": "Chart X", "avLst": ()},
    MSO_SHAPE.CHEVRON: {"basename": "Chevron", "avLst": (("adj", 50000),)},
    MSO_SHAPE.CHORD: {
        "basename": "Chord",
        "avLst": (("adj1", 2700000), ("adj2", 16200000)),
    },
    MSO_SHAPE.CIRCULAR_ARROW: {
        "basename": "Circular Arrow",
        "avLst": (
            ("adj1", 12500),
            ("adj2", 1142319),
            ("adj3", 20457681),
            ("adj4", 10800000),
            ("adj5", 12500),
        ),
    },
    MSO_SHAPE.CLOUD: {"basename": "Cloud", "avLst": ()},
    MSO_SHAPE.CLOUD_CALLOUT: {
        "basename": "Cloud Callout",
        "avLst": (("adj1", -20833), ("adj2", 62500)),
    },
    MSO_SHAPE.CORNER: {
        "basename": "Corner",
        "avLst": (("adj1", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.CORNER_TABS: {"basename": "Corner Tabs", "avLst": ()},
    MSO_SHAPE.CROSS: {"basename": "Cross", "avLst": (("adj", 25000),)},
    MSO_SHAPE.CUBE: {"basename": "Cube", "avLst": (("adj", 25000),)},
    MSO_SHAPE.CURVED_DOWN_ARROW: {
        "basename": "Curved Down Arrow",
        "avLst": (("adj1", 25000), ("adj2", 50000), ("adj3", 25000)),
    },
    MSO_SHAPE.CURVED_DOWN_RIBBON: {
        "basename": "Curved Down Ribbon",
        "avLst": (("adj1", 25000), ("adj2", 50000), ("adj3", 12500)),
    },
    MSO_SHAPE.CURVED_LEFT_ARROW: {
        "basename": "Curved Left Arrow",
        "avLst": (("adj1", 25000), ("adj2", 50000), ("adj3", 25000)),
    },
    MSO_SHAPE.CURVED_RIGHT_ARROW: {
        "basename": "Curved Right Arrow",
        "avLst": (("adj1", 25000), ("adj2", 50000), ("adj3", 25000)),
    },
    MSO_SHAPE.CURVED_UP_ARROW: {
        "basename": "Curved Up Arrow",
        "avLst": (("adj1", 25000), ("adj2", 50000), ("adj3", 25000)),
    },
    MSO_SHAPE.CURVED_UP_RIBBON: {
        "basename": "Curved Up Ribbon",
        "avLst": (("adj1", 25000), ("adj2", 50000), ("adj3", 12500)),
    },
    MSO_SHAPE.DECAGON: {"basename": "Decagon", "avLst": (("vf", 105146),)},
    MSO_SHAPE.DIAGONAL_STRIPE: {
        "basename": "Diagonal Stripe",
        "avLst": (("adj", 50000),),
    },
    MSO_SHAPE.DIAMOND: {"basename": "Diamond", "avLst": ()},
    MSO_SHAPE.DODECAGON: {"basename": "Dodecagon", "avLst": ()},
    MSO_SHAPE.DONUT: {"basename": "Donut", "avLst": (("adj", 25000),)},
    MSO_SHAPE.DOUBLE_BRACE: {"basename": "Double Brace", "avLst": (("adj", 8333),)},
    MSO_SHAPE.DOUBLE_BRACKET: {
        "basename": "Double Bracket",
        "avLst": (("adj", 16667),),
    },
    MSO_SHAPE.DOUBLE_WAVE: {
        "basename": "Double Wave",
        "avLst": (("adj1", 6250), ("adj2", 0)),
    },
    MSO_SHAPE.DOWN_ARROW: {
        "basename": "Down Arrow",
        "avLst": (("adj1", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.DOWN_ARROW_CALLOUT: {
        "basename": "Down Arrow Callout",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000), ("adj4", 64977)),
    },
    MSO_SHAPE.DOWN_RIBBON: {
        "basename": "Down Ribbon",
        "avLst": (("adj1", 16667), ("adj2", 50000)),
    },
    MSO_SHAPE.EXPLOSION1: {"basename": "Explosion", "avLst": ()},
    MSO_SHAPE.EXPLOSION2: {"basename": "Explosion", "avLst": ()},
    MSO_SHAPE.FLOWCHART_ALTERNATE_PROCESS: {
        "basename": "Alternate process",
        "avLst": (),
    },
    MSO_SHAPE.FLOWCHART_CARD: {"basename": "Card", "avLst": ()},
    MSO_SHAPE.FLOWCHART_COLLATE: {"basename": "Collate", "avLst": ()},
    MSO_SHAPE.FLOWCHART_CONNECTOR: {"basename": "Connector", "avLst": ()},
    MSO_SHAPE.FLOWCHART_DATA: {"basename": "Data", "avLst": ()},
    MSO_SHAPE.FLOWCHART_DECISION: {"basename": "Decision", "avLst": ()},
    MSO_SHAPE.FLOWCHART_DELAY: {"basename": "Delay", "avLst": ()},
    MSO_SHAPE.FLOWCHART_DIRECT_ACCESS_STORAGE: {
        "basename": "Direct Access Storage",
        "avLst": (),
    },
    MSO_SHAPE.FLOWCHART_DISPLAY: {"basename": "Display", "avLst": ()},
    MSO_SHAPE.FLOWCHART_DOCUMENT: {"basename": "Document", "avLst": ()},
    MSO_SHAPE.FLOWCHART_EXTRACT: {"basename": "Extract", "avLst": ()},
    MSO_SHAPE.FLOWCHART_INTERNAL_STORAGE: {"basename": "Internal Storage", "avLst": ()},
    MSO_SHAPE.FLOWCHART_MAGNETIC_DISK: {"basename": "Magnetic Disk", "avLst": ()},
    MSO_SHAPE.FLOWCHART_MANUAL_INPUT: {"basename": "Manual Input", "avLst": ()},
    MSO_SHAPE.FLOWCHART_MANUAL_OPERATION: {"basename": "Manual Operation", "avLst": ()},
    MSO_SHAPE.FLOWCHART_MERGE: {"basename": "Merge", "avLst": ()},
    MSO_SHAPE.FLOWCHART_MULTIDOCUMENT: {"basename": "Multidocument", "avLst": ()},
    MSO_SHAPE.FLOWCHART_OFFLINE_STORAGE: {"basename": "Offline Storage", "avLst": ()},
    MSO_SHAPE.FLOWCHART_OFFPAGE_CONNECTOR: {
        "basename": "Off-page Connector",
        "avLst": (),
    },
    MSO_SHAPE.FLOWCHART_OR: {"basename": "Or", "avLst": ()},
    MSO_SHAPE.FLOWCHART_PREDEFINED_PROCESS: {
        "basename": "Predefined Process",
        "avLst": (),
    },
    MSO_SHAPE.FLOWCHART_PREPARATION: {"basename": "Preparation", "avLst": ()},
    MSO_SHAPE.FLOWCHART_PROCESS: {"basename": "Process", "avLst": ()},
    MSO_SHAPE.FLOWCHART_PUNCHED_TAPE: {"basename": "Punched Tape", "avLst": ()},
    MSO_SHAPE.FLOWCHART_SEQUENTIAL_ACCESS_STORAGE: {
        "basename": "Sequential Access Storage",
        "avLst": (),
    },
    MSO_SHAPE.FLOWCHART_SORT: {"basename": "Sort", "avLst": ()},
    MSO_SHAPE.FLOWCHART_STORED_DATA: {"basename": "Stored Data", "avLst": ()},
    MSO_SHAPE.FLOWCHART_SUMMING_JUNCTION: {"basename": "Summing Junction", "avLst": ()},
    MSO_SHAPE.FLOWCHART_TERMINATOR: {"basename": "Terminator", "avLst": ()},
    MSO_SHAPE.FOLDED_CORNER: {"basename": "Folded Corner", "avLst": ()},
    MSO_SHAPE.FRAME: {"basename": "Frame", "avLst": (("adj1", 12500),)},
    MSO_SHAPE.FUNNEL: {"basename": "Funnel", "avLst": ()},
    MSO_SHAPE.GEAR_6: {
        "basename": "Gear 6",
        "avLst": (("adj1", 15000), ("adj2", 3526)),
    },
    MSO_SHAPE.GEAR_9: {
        "basename": "Gear 9",
        "avLst": (("adj1", 10000), ("adj2", 1763)),
    },
    MSO_SHAPE.HALF_FRAME: {
        "basename": "Half Frame",
        "avLst": (("adj1", 33333), ("adj2", 33333)),
    },
    MSO_SHAPE.HEART: {"basename": "Heart", "avLst": ()},
    MSO_SHAPE.HEPTAGON: {
        "basename": "Heptagon",
        "avLst": (("hf", 102572), ("vf", 105210)),
    },
    MSO_SHAPE.HEXAGON: {
        "basename": "Hexagon",
        "avLst": (("adj", 25000), ("vf", 115470)),
    },
    MSO_SHAPE.HORIZONTAL_SCROLL: {
        "basename": "Horizontal Scroll",
        "avLst": (("adj", 12500),),
    },
    MSO_SHAPE.ISOSCELES_TRIANGLE: {
        "basename": "Isosceles Triangle",
        "avLst": (("adj", 50000),),
    },
    MSO_SHAPE.LEFT_ARROW: {
        "basename": "Left Arrow",
        "avLst": (("adj1", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.LEFT_ARROW_CALLOUT: {
        "basename": "Left Arrow Callout",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000), ("adj4", 64977)),
    },
    MSO_SHAPE.LEFT_BRACE: {
        "basename": "Left Brace",
        "avLst": (("adj1", 8333), ("adj2", 50000)),
    },
    MSO_SHAPE.LEFT_BRACKET: {"basename": "Left Bracket", "avLst": (("adj", 8333),)},
    MSO_SHAPE.LEFT_CIRCULAR_ARROW: {
        "basename": "Left Circular Arrow",
        "avLst": (
            ("adj1", 12500),
            ("adj2", -1142319),
            ("adj3", 1142319),
            ("adj4", 10800000),
            ("adj5", 12500),
        ),
    },
    MSO_SHAPE.LEFT_RIGHT_ARROW: {
        "basename": "Left-Right Arrow",
        "avLst": (("adj1", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.LEFT_RIGHT_ARROW_CALLOUT: {
        "basename": "Left-Right Arrow Callout",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000), ("adj4", 48123)),
    },
    MSO_SHAPE.LEFT_RIGHT_CIRCULAR_ARROW: {
        "basename": "Left Right Circular Arrow",
        "avLst": (
            ("adj1", 12500),
            ("adj2", 1142319),
            ("adj3", 20457681),
            ("adj4", 11942319),
            ("adj5", 12500),
        ),
    },
    MSO_SHAPE.LEFT_RIGHT_RIBBON: {
        "basename": "Left Right Ribbon",
        "avLst": (("adj1", 50000), ("adj2", 50000), ("adj3", 16667)),
    },
    MSO_SHAPE.LEFT_RIGHT_UP_ARROW: {
        "basename": "Left-Right-Up Arrow",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000)),
    },
    MSO_SHAPE.LEFT_UP_ARROW: {
        "basename": "Left-Up Arrow",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000)),
    },
    MSO_SHAPE.LIGHTNING_BOLT: {"basename": "Lightning Bolt", "avLst": ()},
    MSO_SHAPE.LINE_CALLOUT_1: {
        "basename": "Line Callout 1",
        "avLst": (("adj1", 18750), ("adj2", -8333), ("adj3", 112500), ("adj4", -38333)),
    },
    MSO_SHAPE.LINE_CALLOUT_1_ACCENT_BAR: {
        "basename": "Line Callout 1 (Accent Bar)",
        "avLst": (("adj1", 18750), ("adj2", -8333), ("adj3", 112500), ("adj4", -38333)),
    },
    MSO_SHAPE.LINE_CALLOUT_1_BORDER_AND_ACCENT_BAR: {
        "basename": "Line Callout 1 (Border and Accent Bar)",
        "avLst": (("adj1", 18750), ("adj2", -8333), ("adj3", 112500), ("adj4", -38333)),
    },
    MSO_SHAPE.LINE_CALLOUT_1_NO_BORDER: {
        "basename": "Line Callout 1 (No Border)",
        "avLst": (("adj1", 18750), ("adj2", -8333), ("adj3", 112500), ("adj4", -38333)),
    },
    MSO_SHAPE.LINE_CALLOUT_2: {
        "basename": "Line Callout 2",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 112500),
            ("adj6", -46667),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_2_ACCENT_BAR: {
        "basename": "Line Callout 2 (Accent Bar)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 112500),
            ("adj6", -46667),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_2_BORDER_AND_ACCENT_BAR: {
        "basename": "Line Callout 2 (Border and Accent Bar)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 112500),
            ("adj6", -46667),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_2_NO_BORDER: {
        "basename": "Line Callout 2 (No Border)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 112500),
            ("adj6", -46667),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_3: {
        "basename": "Line Callout 3",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 100000),
            ("adj6", -16667),
            ("adj7", 112963),
            ("adj8", -8333),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_3_ACCENT_BAR: {
        "basename": "Line Callout 3 (Accent Bar)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 100000),
            ("adj6", -16667),
            ("adj7", 112963),
            ("adj8", -8333),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_3_BORDER_AND_ACCENT_BAR: {
        "basename": "Line Callout 3 (Border and Accent Bar)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 100000),
            ("adj6", -16667),
            ("adj7", 112963),
            ("adj8", -8333),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_3_NO_BORDER: {
        "basename": "Line Callout 3 (No Border)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 100000),
            ("adj6", -16667),
            ("adj7", 112963),
            ("adj8", -8333),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_4: {
        "basename": "Line Callout 3",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 100000),
            ("adj6", -16667),
            ("adj7", 112963),
            ("adj8", -8333),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_4_ACCENT_BAR: {
        "basename": "Line Callout 3 (Accent Bar)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 100000),
            ("adj6", -16667),
            ("adj7", 112963),
            ("adj8", -8333),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_4_BORDER_AND_ACCENT_BAR: {
        "basename": "Line Callout 3 (Border and Accent Bar)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 100000),
            ("adj6", -16667),
            ("adj7", 112963),
            ("adj8", -8333),
        ),
    },
    MSO_SHAPE.LINE_CALLOUT_4_NO_BORDER: {
        "basename": "Line Callout 3 (No Border)",
        "avLst": (
            ("adj1", 18750),
            ("adj2", -8333),
            ("adj3", 18750),
            ("adj4", -16667),
            ("adj5", 100000),
            ("adj6", -16667),
            ("adj7", 112963),
            ("adj8", -8333),
        ),
    },
    MSO_SHAPE.LINE_INVERSE: {"basename": "Straight Connector", "avLst": ()},
    MSO_SHAPE.MATH_DIVIDE: {
        "basename": "Division",
        "avLst": (("adj1", 23520), ("adj2", 5880), ("adj3", 11760)),
    },
    MSO_SHAPE.MATH_EQUAL: {
        "basename": "Equal",
        "avLst": (("adj1", 23520), ("adj2", 11760)),
    },
    MSO_SHAPE.MATH_MINUS: {"basename": "Minus", "avLst": (("adj1", 23520),)},
    MSO_SHAPE.MATH_MULTIPLY: {"basename": "Multiply", "avLst": (("adj1", 23520),)},
    MSO_SHAPE.MATH_NOT_EQUAL: {
        "basename": "Not Equal",
        "avLst": (("adj1", 23520), ("adj2", 6600000), ("adj3", 11760)),
    },
    MSO_SHAPE.MATH_PLUS: {"basename": "Plus", "avLst": (("adj1", 23520),)},
    MSO_SHAPE.MOON: {"basename": "Moon", "avLst": (("adj", 50000),)},
    MSO_SHAPE.NON_ISOSCELES_TRAPEZOID: {
        "basename": "Non-isosceles Trapezoid",
        "avLst": (("adj1", 25000), ("adj2", 25000)),
    },
    MSO_SHAPE.NOTCHED_RIGHT_ARROW: {
        "basename": "Notched Right Arrow",
        "avLst": (("adj1", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.NO_SYMBOL: {"basename": '"No" symbol', "avLst": (("adj", 18750),)},
    MSO_SHAPE.OCTAGON: {"basename": "Octagon", "avLst": (("adj", 29289),)},
    MSO_SHAPE.OVAL: {"basename": "Oval", "avLst": ()},
    MSO_SHAPE.OVAL_CALLOUT: {
        "basename": "Oval Callout",
        "avLst": (("adj1", -20833), ("adj2", 62500)),
    },
    MSO_SHAPE.PARALLELOGRAM: {"basename": "Parallelogram", "avLst": (("adj", 25000),)},
    MSO_SHAPE.PENTAGON: {"basename": "Pentagon", "avLst": (("adj", 50000),)},
    MSO_SHAPE.PIE: {"basename": "Pie", "avLst": (("adj1", 0), ("adj2", 16200000))},
    MSO_SHAPE.PIE_WEDGE: {"basename": "Pie", "avLst": ()},
    MSO_SHAPE.PLAQUE: {"basename": "Plaque", "avLst": (("adj", 16667),)},
    MSO_SHAPE.PLAQUE_TABS: {"basename": "Plaque Tabs", "avLst": ()},
    MSO_SHAPE.QUAD_ARROW: {
        "basename": "Quad Arrow",
        "avLst": (("adj1", 22500), ("adj2", 22500), ("adj3", 22500)),
    },
    MSO_SHAPE.QUAD_ARROW_CALLOUT: {
        "basename": "Quad Arrow Callout",
        "avLst": (("adj1", 18515), ("adj2", 18515), ("adj3", 18515), ("adj4", 48123)),
    },
    MSO_SHAPE.RECTANGLE: {"basename": "Rectangle", "avLst": ()},
    MSO_SHAPE.RECTANGULAR_CALLOUT: {
        "basename": "Rectangular Callout",
        "avLst": (("adj1", -20833), ("adj2", 62500)),
    },
    MSO_SHAPE.REGULAR_PENTAGON: {
        "basename": "Regular Pentagon",
        "avLst": (("hf", 105146), ("vf", 110557)),
    },
    MSO_SHAPE.RIGHT_ARROW: {
        "basename": "Right Arrow",
        "avLst": (("adj1", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.RIGHT_ARROW_CALLOUT: {
        "basename": "Right Arrow Callout",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000), ("adj4", 64977)),
    },
    MSO_SHAPE.RIGHT_BRACE: {
        "basename": "Right Brace",
        "avLst": (("adj1", 8333), ("adj2", 50000)),
    },
    MSO_SHAPE.RIGHT_BRACKET: {"basename": "Right Bracket", "avLst": (("adj", 8333),)},
    MSO_SHAPE.RIGHT_TRIANGLE: {"basename": "Right Triangle", "avLst": ()},
    MSO_SHAPE.ROUNDED_RECTANGLE: {
        "basename": "Rounded Rectangle",
        "avLst": (("adj", 16667),),
    },
    MSO_SHAPE.ROUNDED_RECTANGULAR_CALLOUT: {
        "basename": "Rounded Rectangular Callout",
        "avLst": (("adj1", -20833), ("adj2", 62500), ("adj3", 16667)),
    },
    MSO_SHAPE.ROUND_1_RECTANGLE: {
        "basename": "Round Single Corner Rectangle",
        "avLst": (("adj", 16667),),
    },
    MSO_SHAPE.ROUND_2_DIAG_RECTANGLE: {
        "basename": "Round Diagonal Corner Rectangle",
        "avLst": (("adj1", 16667), ("adj2", 0)),
    },
    MSO_SHAPE.ROUND_2_SAME_RECTANGLE: {
        "basename": "Round Same Side Corner Rectangle",
        "avLst": (("adj1", 16667), ("adj2", 0)),
    },
    MSO_SHAPE.SMILEY_FACE: {"basename": "Smiley Face", "avLst": (("adj", 4653),)},
    MSO_SHAPE.SNIP_1_RECTANGLE: {
        "basename": "Snip Single Corner Rectangle",
        "avLst": (("adj", 16667),),
    },
    MSO_SHAPE.SNIP_2_DIAG_RECTANGLE: {
        "basename": "Snip Diagonal Corner Rectangle",
        "avLst": (("adj1", 0), ("adj2", 16667)),
    },
    MSO_SHAPE.SNIP_2_SAME_RECTANGLE: {
        "basename": "Snip Same Side Corner Rectangle",
        "avLst": (("adj1", 16667), ("adj2", 0)),
    },
    MSO_SHAPE.SNIP_ROUND_RECTANGLE: {
        "basename": "Snip and Round Single Corner Rectangle",
        "avLst": (("adj1", 16667), ("adj2", 16667)),
    },
    MSO_SHAPE.SQUARE_TABS: {"basename": "Square Tabs", "avLst": ()},
    MSO_SHAPE.STAR_10_POINT: {
        "basename": "10-Point Star",
        "avLst": (("adj", 42533), ("hf", 105146)),
    },
    MSO_SHAPE.STAR_12_POINT: {"basename": "12-Point Star", "avLst": (("adj", 37500),)},
    MSO_SHAPE.STAR_16_POINT: {"basename": "16-Point Star", "avLst": (("adj", 37500),)},
    MSO_SHAPE.STAR_24_POINT: {"basename": "24-Point Star", "avLst": (("adj", 37500),)},
    MSO_SHAPE.STAR_32_POINT: {"basename": "32-Point Star", "avLst": (("adj", 37500),)},
    MSO_SHAPE.STAR_4_POINT: {"basename": "4-Point Star", "avLst": (("adj", 12500),)},
    MSO_SHAPE.STAR_5_POINT: {
        "basename": "5-Point Star",
        "avLst": (("adj", 19098), ("hf", 105146), ("vf", 110557)),
    },
    MSO_SHAPE.STAR_6_POINT: {
        "basename": "6-Point Star",
        "avLst": (("adj", 28868), ("hf", 115470)),
    },
    MSO_SHAPE.STAR_7_POINT: {
        "basename": "7-Point Star",
        "avLst": (("adj", 34601), ("hf", 102572), ("vf", 105210)),
    },
    MSO_SHAPE.STAR_8_POINT: {"basename": "8-Point Star", "avLst": (("adj", 37500),)},
    MSO_SHAPE.STRIPED_RIGHT_ARROW: {
        "basename": "Striped Right Arrow",
        "avLst": (("adj1", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.SUN: {"basename": "Sun", "avLst": (("adj", 25000),)},
    MSO_SHAPE.SWOOSH_ARROW: {
        "basename": "Swoosh Arrow",
        "avLst": (("adj1", 25000), ("adj2", 16667)),
    },
    MSO_SHAPE.TEAR: {"basename": "Teardrop", "avLst": (("adj", 100000),)},
    MSO_SHAPE.TRAPEZOID: {"basename": "Trapezoid", "avLst": (("adj", 25000),)},
    MSO_SHAPE.UP_ARROW: {
        "basename": "Up Arrow",
        "avLst": (("adj1", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.UP_ARROW_CALLOUT: {
        "basename": "Up Arrow Callout",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000), ("adj4", 64977)),
    },
    MSO_SHAPE.UP_DOWN_ARROW: {
        "basename": "Up-Down Arrow",
        "avLst": (("adj1", 50000), ("adj1", 50000), ("adj2", 50000), ("adj2", 50000)),
    },
    MSO_SHAPE.UP_DOWN_ARROW_CALLOUT: {
        "basename": "Up-Down Arrow Callout",
        "avLst": (("adj1", 25000), ("adj2", 25000), ("adj3", 25000), ("adj4", 48123)),
    },
    MSO_SHAPE.UP_RIBBON: {
        "basename": "Up Ribbon",
        "avLst": (("adj1", 16667), ("adj2", 50000)),
    },
    MSO_SHAPE.U_TURN_ARROW: {
        "basename": "U-Turn Arrow",
        "avLst": (
            ("adj1", 25000),
            ("adj2", 25000),
            ("adj3", 25000),
            ("adj4", 43750),
            ("adj5", 75000),
        ),
    },
    MSO_SHAPE.VERTICAL_SCROLL: {
        "basename": "Vertical Scroll",
        "avLst": (("adj", 12500),),
    },
    MSO_SHAPE.WAVE: {"basename": "Wave", "avLst": (("adj1", 12500), ("adj2", 0))},
}



# encoding: utf-8

"""Shapes based on the `p:pic` element, including Picture and Movie."""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.dml.line import LineFormat
from pptx.enum.shapes import MSO_SHAPE, MSO_SHAPE_TYPE, PP_MEDIA_TYPE
from pptx.shapes.base import BaseShape
from pptx.shared import ParentedElementProxy
from pptx.util import lazyproperty


class _BasePicture(BaseShape):
    """Base class for shapes based on a `p:pic` element."""

    def __init__(self, pic, parent):
        super(_BasePicture, self).__init__(pic, parent)
        self._pic = pic

    @property
    def crop_bottom(self):
        """|float| representing relative portion cropped from shape bottom.

        Read/write. 1.0 represents 100%. For example, 25% is represented by
        0.25. Negative values are valid as are values greater than 1.0.
        """
        return self._element.srcRect_b

    @crop_bottom.setter
    def crop_bottom(self, value):
        self._element.srcRect_b = value

    @property
    def crop_left(self):
        """|float| representing relative portion cropped from left of shape.

        Read/write. 1.0 represents 100%. A negative value extends the side
        beyond the image boundary.
        """
        return self._element.srcRect_l

    @crop_left.setter
    def crop_left(self, value):
        self._element.srcRect_l = value

    @property
    def crop_right(self):
        """|float| representing relative portion cropped from right of shape.

        Read/write. 1.0 represents 100%.
        """
        return self._element.srcRect_r

    @crop_right.setter
    def crop_right(self, value):
        self._element.srcRect_r = value

    @property
    def crop_top(self):
        """|float| representing relative portion cropped from shape top.

        Read/write. 1.0 represents 100%.
        """
        return self._element.srcRect_t

    @crop_top.setter
    def crop_top(self, value):
        self._element.srcRect_t = value

    def get_or_add_ln(self):
        """
        Return the `a:ln` element containing the line format properties XML
        for this `p:pic`-based shape.
        """
        return self._pic.get_or_add_ln()

    @lazyproperty
    def line(self):
        """
        An instance of |LineFormat|, providing access to the properties of
        the outline bordering this shape, such as its color and width.
        """
        return LineFormat(self)

    @property
    def ln(self):
        """
        The ``<a:ln>`` element containing the line format properties such as
        line color and width. |None| if no ``<a:ln>`` element is present.
        """
        return self._pic.ln


class Movie(_BasePicture):
    """A movie shape, one that places a video on a slide.

    Like |Picture|, a movie shape is based on the `p:pic` element. A movie is
    composed of a video and a *poster frame*, the placeholder image that
    represents the video before it is played.
    """

    @lazyproperty
    def media_format(self):
        """The |_MediaFormat| object for this movie.

        The |_MediaFormat| object provides access to formatting properties
        for the movie.
        """
        return _MediaFormat(self._element, self)

    @property
    def media_type(self):
        """Member of :ref:`PpMediaType` describing this shape.

        The return value is unconditionally `PP_MEDIA_TYPE.MOVIE` in this
        case.
        """
        return PP_MEDIA_TYPE.MOVIE

    @property
    def poster_frame(self):
        """Return |Image| object containing poster frame for this movie.

        Returns |None| if this movie has no poster frame (uncommon).
        """
        slide_part, rId = self.part, self._element.blip_rId
        if rId is None:
            return None
        return slide_part.get_image(rId)

    @property
    def shape_type(self):
        """Return member of :ref:`MsoShapeType` describing this shape.

        The return value is unconditionally ``MSO_SHAPE_TYPE.MEDIA`` in this
        case.
        """
        return MSO_SHAPE_TYPE.MEDIA


class Picture(_BasePicture):
    """A picture shape, one that places an image on a slide.

    Based on the `p:pic` element.
    """

    @property
    def auto_shape_type(self):
        """Member of MSO_SHAPE indicating masking shape.

        A picture can be masked by any of the so-called "auto-shapes"
        available in PowerPoint, such as an ellipse or triangle. When
        a picture is masked by a shape, the shape assumes the same dimensions
        as the picture and the portion of the picture outside the shape
        boundaries does not appear. Note the default value for
        a newly-inserted picture is `MSO_AUTO_SHAPE_TYPE.RECTANGLE`, which
        performs no cropping because the extents of the rectangle exactly
        correspond to the extents of the picture.

        The available shapes correspond to the members of
        :ref:`MsoAutoShapeType`.

        The return value can also be |None|, indicating the picture either
        has no geometry (not expected) or has custom geometry, like
        a freeform shape. A picture with no geometry will have no visible
        representation on the slide, although it can be selected. This is
        because without geometry, there is no "inside-the-shape" for it to
        appear in.
        """
        prstGeom = self._pic.spPr.prstGeom
        if prstGeom is None:  # ---generally means cropped with freeform---
            return None
        return prstGeom.prst

    @auto_shape_type.setter
    def auto_shape_type(self, member):
        MSO_SHAPE.validate(member)
        spPr = self._pic.spPr
        prstGeom = spPr.prstGeom
        if prstGeom is None:
            spPr._remove_custGeom()
            prstGeom = spPr._add_prstGeom()
        prstGeom.prst = member

    @property
    def image(self):
        """
        An |Image| object providing access to the properties and bytes of the
        image in this picture shape.
        """
        slide_part, rId = self.part, self._element.blip_rId
        if rId is None:
            raise ValueError("no embedded image")
        return slide_part.get_image(rId)

    @property
    def shape_type(self):
        """
        Unique integer identifying the type of this shape, unconditionally
        ``MSO_SHAPE_TYPE.PICTURE`` in this case.
        """
        return MSO_SHAPE_TYPE.PICTURE


class _MediaFormat(ParentedElementProxy):
    """Provides access to formatting properties for a Media object.

    Media format properties are things like start point, volume, and
    compression type.
    """



# encoding: utf-8

"""
Test suite for pptx.shapes.placeholder module
"""

from __future__ import absolute_import, print_function, unicode_literals

import pytest

from pptx.chart.data import ChartData
from pptx.enum.shapes import MSO_SHAPE_TYPE, PP_PLACEHOLDER
from pptx.oxml.shapes.shared import ST_Direction, ST_PlaceholderSize
from pptx.parts.image import ImagePart
from pptx.parts.slide import NotesSlidePart, SlideLayoutPart, SlidePart
from pptx.shapes.placeholder import (
    BasePlaceholder,
    _BaseSlidePlaceholder,
    ChartPlaceholder,
    _InheritsDimensions,
    LayoutPlaceholder,
    MasterPlaceholder,
    NotesSlidePlaceholder,
    PicturePlaceholder,
    PlaceholderGraphicFrame,
    PlaceholderPicture,
    TablePlaceholder,
)
from pptx.shapes.shapetree import NotesSlidePlaceholders
from pptx.slide import NotesMaster, SlideLayout, SlideMaster

from ..oxml.unitdata.shape import (
    a_graphicFrame,
    a_ph,
    an_nvGraphicFramePr,
    an_nvPicPr,
    an_nvPr,
    an_nvSpPr,
    an_sp,
)
from ..unitutil.cxml import element, xml
from ..unitutil.file import snippet_seq
from ..unitutil.mock import class_mock, instance_mock, method_mock, property_mock


class Describe_BaseSlidePlaceholder(object):
    def it_knows_its_shape_type(self):
        placeholder = _BaseSlidePlaceholder(None, None)
        assert placeholder.shape_type == MSO_SHAPE_TYPE.PLACEHOLDER

    def it_provides_override_dimensions_when_present(self, override_fixture):
        placeholder, prop_name, expected_value = override_fixture
        assert getattr(placeholder, prop_name) == expected_value

    def it_provides_inherited_dims_when_no_override(self, inherited_fixture):
        placeholder, prop_name, expected_value = inherited_fixture
        value = getattr(placeholder, prop_name)
        placeholder._inherited_value.assert_called_once_with(prop_name)
        assert value == expected_value

    def it_gets_an_inherited_dim_value_to_help(self, base_val_fixture):
        placeholder, attr_name, expected_value = base_val_fixture
        value = placeholder._inherited_value(attr_name)
        assert value == expected_value

    def it_finds_its_base_placeholder_to_help(self, base_ph_fixture):
        placeholder, layout_, idx, layout_placeholder_ = base_ph_fixture
        base_placeholder = placeholder._base_placeholder
        layout_.placeholders.get.assert_called_once_with(idx=idx)
        assert base_placeholder is layout_placeholder_

    def it_can_override_inherited_dimensions(self, dim_set_fixture):
        placeholder, prop_name, value, expected_xml = dim_set_fixture
        setattr(placeholder, prop_name, value)
        assert placeholder._element.xml == expected_xml

    def it_replaces_a_placeholder_element_to_help(self, replace_fixture):
        placeholder, element, spTree, expected_xml = replace_fixture
        placeholder._replace_placeholder_with(element)
        assert spTree.xml == expected_xml
        assert placeholder._element is None

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def base_ph_fixture(self, request, part_prop_, slide_layout_, layout_placeholder_):
        sp_cxml = "p:sp/p:nvSpPr/p:nvPr/p:ph{type=pic,idx=1}"
        placeholder = _BaseSlidePlaceholder(element(sp_cxml), None)
        part_prop_.return_value.slide_layout = slide_layout_
        slide_layout_.placeholders.get.return_value = layout_placeholder_
        return placeholder, slide_layout_, 1, layout_placeholder_

    @pytest.fixture(params=[(True, 42), (False, None)])
    def base_val_fixture(self, request, _base_placeholder_prop_, layout_placeholder_):
        has_base_placeholder, expected_value = request.param
        placeholder = _BaseSlidePlaceholder(None, None)
        if has_base_placeholder:
            layout_placeholder_.width = expected_value
            _base_placeholder_prop_.return_value = layout_placeholder_
        else:
            _base_placeholder_prop_.return_value = None
        return placeholder, "width", expected_value

    @pytest.fixture(
        params=[
            ("p:sp/p:spPr/a:xfrm", "left", 1, "p:sp/p:spPr/a:xfrm/a:off{x=1,y=0}"),
            ("p:sp/p:spPr/a:xfrm", "top", 2, "p:sp/p:spPr/a:xfrm/a:off{x=0,y=2}"),
            ("p:sp/p:spPr/a:xfrm", "width", 3, "p:sp/p:spPr/a:xfrm/a:ext{cx=3,cy=0}"),
            ("p:sp/p:spPr/a:xfrm", "height", 4, "p:sp/p:spPr/a:xfrm/a:ext{cx=0,cy=4}"),
        ]
    )
    def dim_set_fixture(self, request):
        sp_cxml, prop_name, value, expected_cxml = request.param
        placeholder = _BaseSlidePlaceholder(element(sp_cxml), None)
        expected_xml = xml(expected_cxml)
        return placeholder, prop_name, value, expected_xml

    @pytest.fixture(params=["left", "top", "width", "height"])
    def inherited_fixture(self, request, _inherited_value_):
        prop_name = request.param
        placeholder = _BaseSlidePlaceholder(element("p:sp/p:spPr"), None)
        _inherited_value_.return_value = expected_value = 42
        return placeholder, prop_name, expected_value

    @pytest.fixture(
        params=[
            ("left", "p:sp/p:spPr/a:xfrm/a:off{x=12}", 12),
            ("top", "p:sp/p:spPr/a:xfrm/a:off{y=34}", 34),
            ("width", "p:sp/p:spPr/a:xfrm/a:ext{cx=56}", 56),
            ("height", "p:sp/p:spPr/a:xfrm/a:ext{cy=78}", 78),
        ]
    )
    def override_fixture(self, request):
        prop_name, sp_cxml, value = request.param
        slide_placeholder = _BaseSlidePlaceholder(element(sp_cxml), None)
        return slide_placeholder, prop_name, value

    @pytest.fixture
    def replace_fixture(self):
        spTree = element("p:spTree/p:sp/p:nvSpPr/p:nvPr/p:ph{type=pic,idx=10}")
        pic = element("p:pic/p:nvPicPr/p:nvPr")
        placeholder = _BaseSlidePlaceholder(spTree[0], None)
        expected_xml = xml("p:spTree/p:pic/p:nvPicPr/p:nvPr/p:ph{type=pic,idx=10}")
        return placeholder, pic, spTree, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _base_placeholder_prop_(self, request):
        return property_mock(request, _BaseSlidePlaceholder, "_base_placeholder")

    @pytest.fixture
    def _inherited_value_(self, request):
        return method_mock(request, _BaseSlidePlaceholder, "_inherited_value")

    @pytest.fixture
    def layout_placeholder_(self, request):
        return instance_mock(request, LayoutPlaceholder)

    @pytest.fixture
    def part_prop_(self, request, slide_part_):
        return property_mock(
            request, _BaseSlidePlaceholder, "part", return_value=slide_part_
        )

    @pytest.fixture
    def slide_layout_(self, request):
        return instance_mock(request, SlideLayout)

    @pytest.fixture
    def slide_part_(self, request):
        return instance_mock(request, SlidePart)


class DescribeBasePlaceholder(object):
    def it_knows_its_idx_value(self, idx_fixture):
        placeholder, idx = idx_fixture
        assert placeholder.idx == idx

    def it_knows_its_orient_value(self, orient_fixture):
        placeholder, orient = orient_fixture
        assert placeholder.orient == orient

    def it_raises_on_ph_orient_when_not_a_placeholder(self, orient_raise_fixture):
        shape = orient_raise_fixture
        with pytest.raises(ValueError):
            shape.orient

    def it_knows_its_sz_value(self, sz_fixture):
        placeholder, sz = sz_fixture
        assert placeholder.sz == sz

    def it_knows_its_placeholder_type(self, type_fixture):
        placeholder, ph_type = type_fixture
        assert placeholder.ph_type == ph_type

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("sp", None, "title", 0),
            ("sp", 0, "title", 0),
            ("sp", 3, "body", 3),
            ("pic", 6, "pic", 6),
            ("graphicFrame", 9, "tbl", 9),
        ]
    )
    def idx_fixture(self, request):
        tagname, idx, ph_type, expected_idx = request.param
        shape_elm = self.shape_elm_factory(tagname, ph_type, idx)
        placeholder = BasePlaceholder(shape_elm, None)
        return placeholder, expected_idx

    @pytest.fixture(
        params=[(None, ST_Direction.HORZ), (ST_Direction.VERT, ST_Direction.VERT)]
    )
    def orient_fixture(self, request):
        orient, expected_orient = request.param
        ph_bldr = a_ph()
        if orient is not None:
            ph_bldr.with_orient(orient)
        shape_elm = (
            an_sp()
            .with_nsdecls("p")
            .with_child(an_nvSpPr().with_child(an_nvPr().with_child(ph_bldr)))
        ).element
        placeholder = BasePlaceholder(shape_elm, None)
        return placeholder, expected_orient

    @pytest.fixture
    def orient_raise_fixture(self):
        shape_elm = (
            an_sp().with_nsdecls("p").with_child(an_nvSpPr().with_child(an_nvPr()))
        ).element
        shape = BasePlaceholder(shape_elm, None)
        return shape

    @pytest.fixture(
        params=[
            (None, ST_PlaceholderSize.FULL),
            (ST_PlaceholderSize.HALF, ST_PlaceholderSize.HALF),
        ]
    )
    def sz_fixture(self, request):
        sz, expected_sz = request.param
        ph_bldr = a_ph()
        if sz is not None:
            ph_bldr.with_sz(sz)
        shape_elm = (
            an_sp()
            .with_nsdecls("p")
            .with_child(an_nvSpPr().with_child(an_nvPr().with_child(ph_bldr)))
        ).element
        placeholder = BasePlaceholder(shape_elm, None)
        return placeholder, expected_sz

    @pytest.fixture(
        params=[
            ("sp", None, 1, PP_PLACEHOLDER.OBJECT),
            ("sp", "title", 0, PP_PLACEHOLDER.TITLE),
            ("pic", "pic", 6, PP_PLACEHOLDER.PICTURE),
            ("graphicFrame", "tbl", 9, PP_PLACEHOLDER.TABLE),
        ]
    )
    def type_fixture(self, request):
        tagname, ph_type, idx, expected_ph_type = request.param
        shape_elm = self.shape_elm_factory(tagname, ph_type, idx)
        placeholder = BasePlaceholder(shape_elm, None)
        return placeholder, expected_ph_type

    # fixture components ---------------------------------------------

    @staticmethod
    def shape_elm_factory(tagname, ph_type, idx):
        root_bldr, nvXxPr_bldr = {
            "sp": (an_sp().with_nsdecls("p"), an_nvSpPr()),
            "pic": (an_sp().with_nsdecls("p"), an_nvPicPr()),
            "graphicFrame": (a_graphicFrame().with_nsdecls("p"), an_nvGraphicFramePr()),
        }[tagname]
        ph_bldr = {
            None: a_ph().with_idx(idx),
            "obj": a_ph().with_idx(idx),
            "title": a_ph().with_type("title"),
            "body": a_ph().with_type("body").with_idx(idx),
            "pic": a_ph().with_type("pic").with_idx(idx),
            "tbl": a_ph().with_type("tbl").with_idx(idx),
        }[ph_type]
        return (
            root_bldr.with_child(nvXxPr_bldr.with_child(an_nvPr().with_child(ph_bldr)))
        ).element


class DescribeChartPlaceholder(object):
    def it_can_insert_a_chart_into_itself(self, insert_fixture):
        chart_ph, chart_type, chart_data_, graphicFrame = insert_fixture[:4]
        rId, PlaceholderGraphicFrame_, ph_graphic_frame_ = insert_fixture[4:]

        ph_graphic_frame = chart_ph.insert_chart(chart_type, chart_data_)

        chart_ph.part.add_chart_part.assert_called_once_with(chart_type, chart_data_)
        chart_ph._new_chart_graphicFrame.assert_called_once_with(
            rId, chart_ph.left, chart_ph.top, chart_ph.width, chart_ph.height
        )
        chart_ph._replace_placeholder_with.assert_called_once_with(graphicFrame)
        PlaceholderGraphicFrame_.assert_called_once_with(graphicFrame, chart_ph._parent)
        assert ph_graphic_frame is ph_graphic_frame_

    def it_creates_a_graphicFrame_element_to_help(self, new_fixture):
        chart_ph, rId, x, y, cx, cy, expected_xml = new_fixture
        graphicFrame = chart_ph._new_chart_graphicFrame(rId, x, y, cx, cy)
        assert graphicFrame.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def insert_fixture(
        self,
        part_prop_,
        chart_data_,
        PlaceholderGraphicFrame_,
        placeholder_graphic_frame_,
        _new_chart_graphicFrame_,
        _replace_placeholder_with_,
    ):
        sp_cxml = "p:sp/p:spPr/a:xfrm/(a:off{x=1,y=2},a:ext{cx=3,cy=4})"
        chart_ph = ChartPlaceholder(element(sp_cxml), "parent")
        chart_type, rId, graphicFrame = 42, "rId6", element("p:graphicFrame")
        part_prop_.return_value.add_chart_part.return_value = rId
        _new_chart_graphicFrame_.return_value = graphicFrame
        return (
            chart_ph,
            chart_type,
            chart_data_,
            graphicFrame,
            rId,
            PlaceholderGraphicFrame_,
            placeholder_graphic_frame_,
        )

    @pytest.fixture
    def new_fixture(self):
        sp_cxml = "p:sp/p:nvSpPr/p:cNvPr{id=4,name=bar}"
        chart_ph = ChartPlaceholder(element(sp_cxml), None)
        rId, x, y, cx, cy = "rId42", 1, 2, 3, 4
        expected_xml = snippet_seq("placeholders")[1]
        return chart_ph, rId, x, y, cx, cy, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def chart_data_(self, request):
        return instance_mock(request, ChartData)

    @pytest.fixture
    def _new_chart_graphicFrame_(self, request):
        return method_mock(request, ChartPlaceholder, "_new_chart_graphicFrame")

    @pytest.fixture
    def part_prop_(self, request, slide_):
        return property_mock(request, ChartPlaceholder, "part", return_value=slide_)

    @pytest.fixture
    def PlaceholderGraphicFrame_(self, request, placeholder_graphic_frame_):
        return class_mock(
            request,
            "pptx.shapes.placeholder.PlaceholderGraphicFrame",
            return_value=placeholder_graphic_frame_,
        )

    @pytest.fixture
    def placeholder_graphic_frame_(self, request):
        return instance_mock(request, PlaceholderGraphicFrame)

    @pytest.fixture
    def _replace_placeholder_with_(self, request):
        return method_mock(request, ChartPlaceholder, "_replace_placeholder_with")

    @pytest.fixture
    def slide_(self, request):
        return instance_mock(request, SlidePart)


class DescribeLayoutPlaceholder(object):
    def it_uses_InheritsDimensions_mixin(self):
        layout_placeholder = LayoutPlaceholder(None, None)
        assert isinstance(layout_placeholder, _InheritsDimensions)

    def it_finds_its_base_placeholder_to_help(self, base_ph_fixture):
        layout_placeholder, master_, mstr_ph_type, master_placeholder_ = base_ph_fixture
        master_placeholder = layout_placeholder._base_placeholder
        master_.placeholders.get.assert_called_once_with(mstr_ph_type, None)
        assert master_placeholder is master_placeholder_

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("p:sp/p:nvSpPr/p:nvPr/p:ph{type=body}", PP_PLACEHOLDER.BODY),
            ("p:sp/p:nvSpPr/p:nvPr/p:ph{type=tbl}", PP_PLACEHOLDER.BODY),
            ("p:sp/p:nvSpPr/p:nvPr/p:ph{type=title}", PP_PLACEHOLDER.TITLE),
        ]
    )
    def base_ph_fixture(self, request, slide_master_, master_placeholder_, part_prop_):
        sp_cxml, mstr_ph_type = request.param
        sp = element(sp_cxml)
        layout_placeholder = LayoutPlaceholder(sp, None)
        part_prop_.return_value.slide_master = slide_master_
        slide_master_.placeholders.get.return_value = master_placeholder_
        return (layout_placeholder, slide_master_, mstr_ph_type, master_placeholder_)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def master_placeholder_(self, request):
        return instance_mock(request, MasterPlaceholder)

    @pytest.fixture
    def part_prop_(self, request, slide_layout_part_):
        return property_mock(
            request, LayoutPlaceholder, "part", return_value=slide_layout_part_
        )

    @pytest.fixture
    def slide_layout_part_(self, request):
        return instance_mock(request, SlideLayoutPart)

    @pytest.fixture
    def slide_master_(self, request):
        return instance_mock(request, SlideMaster)


class DescribeNotesSlidePlaceholder(object):
    def it_finds_its_base_placeholder_to_help(self, base_ph_fixture):
        placeholder, notes_master_, ph_type, master_placeholder_ = base_ph_fixture
        base_placeholder = placeholder._base_placeholder
        notes_master_.placeholders.get.assert_called_once_with(ph_type=ph_type)
        assert base_placeholder is master_placeholder_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def base_ph_fixture(
        self,
        request,
        notes_master_,
        master_placeholder_,
        part_prop_,
        notes_slide_part_,
        notes_slide_placeholders_,
    ):
        sp_cxml = "p:sp/p:nvSpPr/p:nvPr/p:ph{type=body}"
        placeholder = NotesSlidePlaceholder(element(sp_cxml), None)
        notes_slide_part_.notes_master = notes_master_
        notes_master_.placeholders = notes_slide_placeholders_
        notes_slide_placeholders_.get.return_value = master_placeholder_
        return (placeholder, notes_master_, PP_PLACEHOLDER.BODY, master_placeholder_)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def notes_slide_placeholders_(self, request):
        return instance_mock(request, NotesSlidePlaceholders)

    @pytest.fixture
    def master_placeholder_(self, request):
        return instance_mock(request, MasterPlaceholder)

    @pytest.fixture
    def notes_master_(self, request):
        return instance_mock(request, NotesMaster)

    @pytest.fixture
    def notes_slide_part_(self, request):
        return instance_mock(request, NotesSlidePart)

    @pytest.fixture
    def part_prop_(self, request, notes_slide_part_):
        return property_mock(
            request, NotesSlidePlaceholder, "part", return_value=notes_slide_part_
        )


class DescribePicturePlaceholder(object):
    def it_can_insert_a_picture_into_itself(self, insert_fixture):
        picture_ph, image_file, pic = insert_fixture[:3]
        PlaceholderPicture_, placeholder_picture_ = insert_fixture[3:]

        placeholder_picture = picture_ph.insert_picture(image_file)

        picture_ph._new_placeholder_pic.assert_called_once_with(image_file)
        picture_ph._replace_placeholder_with.assert_called_once_with(pic)
        PlaceholderPicture_.assert_called_once_with(pic, picture_ph._parent)
        assert placeholder_picture is placeholder_picture_

    def it_creates_a_pic_element_to_help(self, pic_fixture):
        picture_ph, image_file, expected_xml = pic_fixture
        pic = picture_ph._new_placeholder_pic(image_file)
        picture_ph._get_or_add_image.assert_called_once_with(image_file)
        assert pic.xml == expected_xml

    def it_adds_an_image_to_help(self, get_or_add_fixture):
        placeholder, image_file, expected_value = get_or_add_fixture

        value = placeholder._get_or_add_image(image_file)

        placeholder.part.get_or_add_image_part.assert_called_once_with(image_file)
        assert value == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def get_or_add_fixture(self, part_prop_, image_part_):
        placeholder = PicturePlaceholder(None, None)
        image_file, rId, desc, image_size = "f.png", "rId6", "desc", (42, 24)
        part_prop_.return_value.get_or_add_image_part.return_value = (image_part_, rId)
        image_part_.desc, image_part_._px_size = desc, image_size
        expected_value = rId, desc, image_size
        return placeholder, image_file, expected_value

    @pytest.fixture
    def insert_fixture(
        self,
        PlaceholderPicture_,
        placeholder_picture_,
        _new_placeholder_pic_,
        _replace_placeholder_with_,
    ):
        picture_ph = PicturePlaceholder(None, "parent")
        image_file, pic = "foobar.png", element("p:pic")
        _new_placeholder_pic_.return_value = pic
        PlaceholderPicture_.return_value = placeholder_picture_
        return (picture_ph, image_file, pic, PlaceholderPicture_, placeholder_picture_)

    @pytest.fixture(params=[((444, 333), ("l", "r")), ((333, 444), ("t", "b"))])
    def pic_fixture(self, request, _get_or_add_image_):
        image_size, crop_attr_names = request.param
        sp_cxml = (
            "p:sp/(p:nvSpPr/p:cNvPr{id=2,name=foo},p:spPr/a:xfrm/a:ext{cx=99" ",cy=99})"
        )
        sp = element(sp_cxml)
        picture_ph = PicturePlaceholder(sp, None)
        image_file = "foobar.png"
        _get_or_add_image_.return_value = 42, "bar", image_size
        expected_xml = xml(
            "p:pic/(p:nvPicPr/(p:cNvPr{id=2,name=foo,descr=bar},p:cNvPicPr/a"
            ":picLocks{noGrp=1,noChangeAspect=1},p:nvPr),p:blipFill/(a:blip{"
            "r:embed=42},a:srcRect{%s=12500,%s=12500},a:stretch/a:fillRect),"
            "p:spPr)" % crop_attr_names
        )
        return picture_ph, image_file, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _get_or_add_image_(self, request):
        return method_mock(request, PicturePlaceholder, "_get_or_add_image")

    @pytest.fixture
    def image_part_(self, request):
        return instance_mock(request, ImagePart)

    @pytest.fixture
    def _new_placeholder_pic_(self, request):
        return method_mock(request, PicturePlaceholder, "_new_placeholder_pic")

    @pytest.fixture
    def part_prop_(self, request, slide_):
        return property_mock(request, PicturePlaceholder, "part", return_value=slide_)

    @pytest.fixture
    def PlaceholderPicture_(self, request):
        return class_mock(request, "pptx.shapes.placeholder.PlaceholderPicture")

    @pytest.fixture
    def placeholder_picture_(self, request):
        return instance_mock(request, PlaceholderPicture)

    @pytest.fixture
    def _replace_placeholder_with_(self, request):
        return method_mock(request, PicturePlaceholder, "_replace_placeholder_with")

    @pytest.fixture
    def slide_(self, request):
        return instance_mock(request, SlidePart)


class DescribeTablePlaceholder(object):
    def it_can_insert_a_table_into_itself(self, insert_fixture):
        table_ph, rows, cols, graphicFrame = insert_fixture[:4]
        PlaceholderGraphicFrame_, ph_graphic_frame_ = insert_fixture[4:]

        ph_graphic_frame = table_ph.insert_table(rows, cols)

        table_ph._new_placeholder_table.assert_called_once_with(rows, cols)
        table_ph._replace_placeholder_with.assert_called_once_with(graphicFrame)
        PlaceholderGraphicFrame_.assert_called_once_with(graphicFrame, table_ph._parent)
        assert ph_graphic_frame is ph_graphic_frame_

    def it_creates_a_graphicFrame_element_to_help(self, new_fixture):
        table_ph, rows, cols, expected_xml = new_fixture
        graphicFrame = table_ph._new_placeholder_table(rows, cols)
        assert graphicFrame.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def insert_fixture(
        self,
        PlaceholderGraphicFrame_,
        placeholder_graphic_frame_,
        _new_placeholder_table_,
        _replace_placeholder_with_,
    ):
        table_ph = TablePlaceholder(None, "parent")
        rows, cols, graphicFrame = 4, 2, element("p:graphicFrame")
        _new_placeholder_table_.return_value = graphicFrame
        PlaceholderGraphicFrame_.return_value = placeholder_graphic_frame_
        return (
            table_ph,
            rows,
            cols,
            graphicFrame,
            PlaceholderGraphicFrame_,
            placeholder_graphic_frame_,
        )

    @pytest.fixture
    def new_fixture(self):
        sp_cxml = (
            "p:sp/(p:nvSpPr/p:cNvPr{id=2,name=foo},p:spPr/a:xfrm/(a:off{x=1,"
            "y=2},a:ext{cx=3,cy=4}))"
        )
        table_ph = TablePlaceholder(element(sp_cxml), None)
        rows, cols = 1, 1
        expected_xml = snippet_seq("placeholders")[0]
        return table_ph, rows, cols, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _new_placeholder_table_(self, request):
        return method_mock(request, TablePlaceholder, "_new_placeholder_table")

    @pytest.fixture
    def PlaceholderGraphicFrame_(self, request):
        return class_mock(request, "pptx.shapes.placeholder.PlaceholderGraphicFrame")

    @pytest.fixture
    def placeholder_graphic_frame_(self, request):
        return instance_mock(request, PlaceholderGraphicFrame)

    @pytest.fixture
    def _replace_placeholder_with_(self, request):
        return method_mock(request, TablePlaceholder, "_replace_placeholder_with")



# encoding: utf-8

"""DrawingML objects related to line formatting."""

from __future__ import absolute_import, division, print_function, unicode_literals

from ..enum.dml import MSO_FILL
from .fill import FillFormat
from ..util import Emu, lazyproperty


class LineFormat(object):
    """Provides access to line properties such as color, style, and width.

    A LineFormat object is typically accessed via the ``.line`` property of
    a shape such as |Shape| or |Picture|.
    """

    def __init__(self, parent):
        super(LineFormat, self).__init__()
        self._parent = parent

    @lazyproperty
    def color(self):
        """
        The |ColorFormat| instance that provides access to the color settings
        for this line. Essentially a shortcut for ``line.fill.fore_color``.
        As a side-effect, accessing this property causes the line fill type
        to be set to ``MSO_FILL.SOLID``. If this sounds risky for your use
        case, use ``line.fill.type`` to non-destructively discover the
        existing fill type.
        """
        if self.fill.type != MSO_FILL.SOLID:
            self.fill.solid()
        return self.fill.fore_color

    @property
    def dash_style(self):
        """Return value indicating line style.

        Returns a member of :ref:`MsoLineDashStyle` indicating line style, or
        |None| if no explicit value has been set. When no explicit value has
        been set, the line dash style is inherited from the style hierarchy.

        Assigning |None| removes any existing explicitly-defined dash style.
        """
        ln = self._ln
        if ln is None:
            return None
        return ln.prstDash_val

    @dash_style.setter
    def dash_style(self, dash_style):
        if dash_style is None:
            ln = self._ln
            if ln is None:
                return
            ln._remove_prstDash()
            ln._remove_custDash()
            return
        ln = self._get_or_add_ln()
        ln.prstDash_val = dash_style

    @lazyproperty
    def fill(self):
        """
        |FillFormat| instance for this line, providing access to fill
        properties such as foreground color.
        """
        ln = self._get_or_add_ln()
        return FillFormat.from_fill_parent(ln)

    @property
    def width(self):
        """
        The width of the line expressed as an integer number of :ref:`English
        Metric Units <EMU>`. The returned value is an instance of |Length|,
        a value class having properties such as `.inches`, `.cm`, and `.pt`
        for converting the value into convenient units.
        """
        ln = self._ln
        if ln is None:
            return Emu(0)
        return ln.w

    @width.setter
    def width(self, emu):
        if emu is None:
            emu = 0
        ln = self._get_or_add_ln()
        ln.w = emu

    def _get_or_add_ln(self):
        """
        Return the ``<a:ln>`` element containing the line format properties
        in the XML.
        """
        return self._parent.get_or_add_ln()

    @property
    def _ln(self):
        return self._parent.ln



# encoding: utf-8

"""
Series-related oxml objects.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from ...enum.chart import XL_MARKER_STYLE
from ..simpletypes import ST_MarkerSize
from ..xmlchemy import BaseOxmlElement, RequiredAttribute, ZeroOrOne


class CT_Marker(BaseOxmlElement):
    """
    `c:marker` custom element class, containing visual properties for a data
    point marker on line-type charts.
    """

    _tag_seq = ("c:symbol", "c:size", "c:spPr", "c:extLst")
    symbol = ZeroOrOne("c:symbol", successors=_tag_seq[1:])
    size = ZeroOrOne("c:size", successors=_tag_seq[2:])
    spPr = ZeroOrOne("c:spPr", successors=_tag_seq[3:])
    del _tag_seq

    @property
    def size_val(self):
        """
        Return the value of `./c:size/@val`, specifying the size of this
        marker in points. Returns |None| if no `c:size` element is present or
        its val attribute is not present.
        """
        size = self.size
        if size is None:
            return None
        return size.val

    @property
    def symbol_val(self):
        """
        Return the value of `./c:symbol/@val`, specifying the shape of this
        marker. Returns |None| if no `c:symbol` element is present.
        """
        symbol = self.symbol
        if symbol is None:
            return None
        return symbol.val


class CT_MarkerSize(BaseOxmlElement):
    """
    `c:size` custom element class, specifying the size (in points) of a data
    point marker for a line, XY, or radar chart.
    """

    val = RequiredAttribute("val", ST_MarkerSize)


class CT_MarkerStyle(BaseOxmlElement):
    """
    `c:symbol` custom element class, specifying the shape of a data point
    marker for a line, XY, or radar chart.
    """

    val = RequiredAttribute("val", XL_MARKER_STYLE)



# encoding: utf-8

"""Gherkin step implementations for chart axis features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, then, when

from pptx import Presentation
from pptx.enum.chart import XL_AXIS_CROSSES, XL_CATEGORY_TYPE

from helpers import test_pptx


# given ===================================================


@given("a {axis_type} axis")
def given_a_axis_type_axis(context, axis_type):
    prs = Presentation(test_pptx("cht-axis-props"))
    chart = prs.slides[0].shapes[0].chart
    context.axis = {"category": chart.category_axis, "value": chart.value_axis}[
        axis_type
    ]


@given("a major gridlines")
def given_a_major_gridlines(context):
    prs = Presentation(test_pptx("cht-gridlines-props"))
    axis = prs.slides[0].shapes[0].chart.value_axis
    context.gridlines = axis.major_gridlines


@given("a value axis having category axis crossing of {crossing}")
def given_a_value_axis_having_cat_ax_crossing_of(context, crossing):
    slide_idx = {"automatic": 0, "maximum": 2, "minimum": 3, "2.75": 4, "-1.5": 5}[
        crossing
    ]
    prs = Presentation(test_pptx("cht-axis-props"))
    context.value_axis = prs.slides[slide_idx].shapes[0].chart.value_axis


@given("an axis")
def given_an_axis(context):
    prs = Presentation(test_pptx("cht-axis-props"))
    chart = prs.slides[0].shapes[0].chart
    context.axis = chart.value_axis


@given("an axis having {a_or_no} title")
def given_an_axis_having_a_or_no_title(context, a_or_no):
    prs = Presentation(test_pptx("cht-axis-props"))
    chart = prs.slides[7].shapes[0].chart
    context.axis = {"a": chart.value_axis, "no": chart.category_axis}[a_or_no]


@given("an axis having {major_or_minor} gridlines")
def given_an_axis_having_major_or_minor_gridlines(context, major_or_minor):
    prs = Presentation(test_pptx("cht-axis-props"))
    chart = prs.slides[0].shapes[0].chart
    context.axis = chart.value_axis


@given("an axis having {major_or_minor} unit of {value}")
def given_an_axis_having_major_or_minor_unit_of_value(context, major_or_minor, value):
    slide_idx = 0 if value == "Auto" else 1
    prs = Presentation(test_pptx("cht-axis-props"))
    chart = prs.slides[slide_idx].shapes[0].chart
    context.axis = chart.value_axis


@given("an axis of type {cls_name}")
def given_an_axis_of_type_cls_name(context, cls_name):
    slide_idx = {"CategoryAxis": 0, "DateAxis": 6}[cls_name]
    prs = Presentation(test_pptx("cht-axis-props"))
    chart = prs.slides[slide_idx].shapes[0].chart
    context.axis = chart.category_axis


@given("an axis not having {major_or_minor} gridlines")
def given_an_axis_not_having_major_or_minor_gridlines(context, major_or_minor):
    prs = Presentation(test_pptx("cht-axis-props"))
    chart = prs.slides[0].shapes[0].chart
    context.axis = chart.category_axis


@given("an axis title")
def given_an_axis_title(context):
    prs = Presentation(test_pptx("cht-axis-props"))
    context.axis_title = prs.slides[7].shapes[0].chart.value_axis.axis_title


@given("an axis title having {a_or_no} text frame")
def given_an_axis_title_having_a_or_no_text_frame(context, a_or_no):
    prs = Presentation(test_pptx("cht-axis-props"))
    chart = prs.slides[7].shapes[0].chart
    axis = {"a": chart.value_axis, "no": chart.category_axis}[a_or_no]
    context.axis_title = axis.axis_title


@given("tick labels having an offset of {setting}")
def given_tick_labels_having_an_offset_of_setting(context, setting):
    slide_idx = {"no explicit setting": 0, "420": 1}[setting]
    prs = Presentation(test_pptx("cht-ticklabels-props"))
    chart = prs.slides[slide_idx].shapes[0].chart
    context.tick_labels = chart.category_axis.tick_labels


# when ====================================================


@when("I assign {value} to axis.has_title")
def when_I_assign_value_to_axis_has_title(context, value):
    context.axis.has_title = {"True": True, "False": False}[value]


@when("I assign {value} to axis.has_{major_or_minor}_gridlines")
def when_I_assign_value_to_axis_has_major_or_minor_gridlines(
    context, value, major_or_minor
):
    axis = context.axis
    propname = "has_%s_gridlines" % major_or_minor
    new_value = {"True": True, "False": False}[value]
    setattr(axis, propname, new_value)


@when("I assign {value} to axis.{major_or_minor}_unit")
def when_I_assign_value_to_axis_major_or_minor_unit(context, value, major_or_minor):
    axis = context.axis
    propname = "%s_unit" % major_or_minor
    new_value = {"8.4": 8.4, "5": 5, "None": None}[value]
    setattr(axis, propname, new_value)


@when("I assign {value} to axis_title.has_text_frame")
def when_I_assign_value_to_axis_title_has_text_frame(context, value):
    context.axis_title.has_text_frame = {"True": True, "False": False}[value]


@when("I assign {value} to tick_labels.offset")
def when_I_assign_value_to_tick_labels_offset(context, value):
    new_value = int(value)
    context.tick_labels.offset = new_value


@when("I assign {member} to value_axis.crosses")
def when_I_assign_member_to_value_axis_crosses(context, member):
    value_axis = context.value_axis
    value_axis.crosses = getattr(XL_AXIS_CROSSES, member)


@when("I assign {value} to value_axis.crosses_at")
def when_I_assign_value_to_value_axis_crosses_at(context, value):
    new_value = None if value == "None" else float(value)
    context.value_axis.crosses_at = new_value


# then ====================================================


@then("axis.axis_title is an AxisTitle object")
def then_axis_axis_title_is_an_AxisTitle_object(context):
    class_name = type(context.axis.axis_title).__name__
    assert class_name == "AxisTitle", "got %s" % class_name


@then("axis.category_type is XL_CATEGORY_TYPE.{member}")
def then_axis_category_type_is_XL_CATEGORY_TYPE_member(context, member):
    expected_value = getattr(XL_CATEGORY_TYPE, member)
    category_type = context.axis.category_type
    assert category_type is expected_value, "got %s" % category_type


@then("axis.format is a ChartFormat object")
def then_axis_format_is_a_ChartFormat_object(context):
    axis = context.axis
    assert type(axis.format).__name__ == "ChartFormat"


@then("axis.format.fill is a FillFormat object")
def then_axis_format_fill_is_a_FillFormat_object(context):
    axis = context.axis
    assert type(axis.format.fill).__name__ == "FillFormat"


@then("axis.format.line is a LineFormat object")
def then_axis_format_line_is_a_LineFormat_object(context):
    axis = context.axis
    assert type(axis.format.line).__name__ == "LineFormat"


@then("axis.has_title is {value}")
def then_axis_has_title_is_value(context, value):
    axis = context.axis
    actual_value = axis.has_title
    expected_value = {"True": True, "False": False}[value]
    assert actual_value is expected_value, "got %s" % actual_value


@then("axis.has_{major_or_minor}_gridlines is {value}")
def then_axis_has_major_or_minor_gridlines_is_expected_value(
    context, major_or_minor, value
):
    axis = context.axis
    actual_value = {
        "major": axis.has_major_gridlines,
        "minor": axis.has_minor_gridlines,
    }[major_or_minor]
    expected_value = {"True": True, "False": False}[value]
    assert actual_value is expected_value, "got %s" % actual_value


@then("axis.major_gridlines is a MajorGridlines object")
def then_axis_major_gridlines_is_a_MajorGridlines_object(context):
    axis = context.axis
    assert type(axis.major_gridlines).__name__ == "MajorGridlines"


@then("axis.{major_or_minor}_unit is {value}")
def then_axis_major_or_minor_unit_is_value(context, major_or_minor, value):
    axis = context.axis
    propname = "%s_unit" % major_or_minor
    actual_value = getattr(axis, propname)
    expected_value = {"20.0": 20.0, "8.4": 8.4, "5.0": 5.0, "4.2": 4.2, "None": None}[
        value
    ]
    assert actual_value == expected_value, "got %s" % actual_value


@then("axis_title.format is a ChartFormat object")
def then_axis_title_format_is_a_ChartFormat_object(context):
    class_name = type(context.axis_title.format).__name__
    assert class_name == "ChartFormat", "got %s" % class_name


@then("axis_title.format.fill is a FillFormat object")
def then_axis_title_format_fill_is_a_FillFormat_object(context):
    class_name = type(context.axis_title.format.fill).__name__
    assert class_name == "FillFormat", "got %s" % class_name


@then("axis_title.format.line is a LineFormat object")
def then_axis_title_format_line_is_a_LineFormat_object(context):
    class_name = type(context.axis_title.format.line).__name__
    assert class_name == "LineFormat", "got %s" % class_name


@then("axis_title.has_text_frame is {value}")
def then_axis_title_has_text_frame_is_value(context, value):
    actual_value = context.axis_title.has_text_frame
    expected_value = {"True": True, "False": False}[value]
    assert actual_value is expected_value, "got %s" % actual_value


@then("axis_title.text_frame is a TextFrame object")
def then_axis_title_text_frame_is_a_TextFrame_object(context):
    class_name = type(context.axis_title.text_frame).__name__
    assert class_name == "TextFrame", "got %s" % class_name


@then("gridlines.format is a ChartFormat object")
def then_gridlines_format_is_a_ChartFormat_object(context):
    gridlines = context.gridlines
    assert type(gridlines.format).__name__ == "ChartFormat"


@then("gridlines.format.fill is a FillFormat object")
def then_gridlines_format_fill_is_a_FillFormat_object(context):
    gridlines = context.gridlines
    assert type(gridlines.format.fill).__name__ == "FillFormat"


@then("gridlines.format.line is a LineFormat object")
def then_gridlines_format_line_is_a_LineFormat_object(context):
    gridlines = context.gridlines
    assert type(gridlines.format.line).__name__ == "LineFormat"


@then("tick_labels.offset is {value}")
def then_tick_labels_offset_is_expected_value(context, value):
    expected_value = int(value)
    tick_labels = context.tick_labels
    assert tick_labels.offset == expected_value, "got %s" % tick_labels.offset


@then("value_axis.crosses is {member}")
def then_value_axis_crosses_is_value(context, member):
    value_axis = context.value_axis
    expected_value = getattr(XL_AXIS_CROSSES, member)
    assert value_axis.crosses == expected_value, "got %s" % value_axis.crosses


@then("value_axis.crosses_at is {value}")
def then_value_axis_crosses_at_is_value(context, value):
    value_axis = context.value_axis
    expected_value = None if value == "None" else float(value)
    assert value_axis.crosses_at == expected_value, "got %s" % value_axis.crosses_at



# encoding: utf-8

"""
DrawingML objects related to color, ColorFormat being the most prominent.
"""

from __future__ import absolute_import, print_function, unicode_literals

from ..enum.dml import MSO_COLOR_TYPE, MSO_THEME_COLOR
from ..oxml.dml.color import (
    CT_HslColor,
    CT_PresetColor,
    CT_SchemeColor,
    CT_ScRgbColor,
    CT_SRgbColor,
    CT_SystemColor,
)


class ColorFormat(object):
    """
    Provides access to color settings such as RGB color, theme color, and
    luminance adjustments.
    """

    def __init__(self, eg_colorChoice_parent, color):
        super(ColorFormat, self).__init__()
        self._xFill = eg_colorChoice_parent
        self._color = color

    @property
    def brightness(self):
        """
        Read/write float value between -1.0 and 1.0 indicating the brightness
        adjustment for this color, e.g. -0.25 is 25% darker and 0.4 is 40%
        lighter. 0 means no brightness adjustment.
        """
        return self._color.brightness

    @brightness.setter
    def brightness(self, value):
        self._validate_brightness_value(value)
        self._color.brightness = value

    @classmethod
    def from_colorchoice_parent(cls, eg_colorChoice_parent):
        xClr = eg_colorChoice_parent.eg_colorChoice
        color = _Color(xClr)
        color_format = cls(eg_colorChoice_parent, color)
        return color_format

    @property
    def rgb(self):
        """
        |RGBColor| value of this color, or None if no RGB color is explicitly
        defined for this font. Setting this value to an |RGBColor| instance
        causes its type to change to MSO_COLOR_TYPE.RGB. If the color was a
        theme color with a brightness adjustment, the brightness adjustment
        is removed when changing it to an RGB color.
        """
        return self._color.rgb

    @rgb.setter
    def rgb(self, rgb):
        if not isinstance(rgb, RGBColor):
            raise ValueError("assigned value must be type RGBColor")
        # change to rgb color format if not already
        if not isinstance(self._color, _SRgbColor):
            srgbClr = self._xFill.get_or_change_to_srgbClr()
            self._color = _SRgbColor(srgbClr)
        # call _SRgbColor instance to do the setting
        self._color.rgb = rgb

    @property
    def theme_color(self):
        """Theme color value of this color.

        Value is a member of :ref:`MsoThemeColorIndex`, e.g.
        ``MSO_THEME_COLOR.ACCENT_1``. Raises AttributeError on access if the
        color is not type ``MSO_COLOR_TYPE.SCHEME``. Assigning a member of
        :ref:`MsoThemeColorIndex` causes the color's type to change to
        ``MSO_COLOR_TYPE.SCHEME``.
        """
        return self._color.theme_color

    @theme_color.setter
    def theme_color(self, mso_theme_color_idx):
        # change to theme color format if not already
        if not isinstance(self._color, _SchemeColor):
            schemeClr = self._xFill.get_or_change_to_schemeClr()
            self._color = _SchemeColor(schemeClr)
        self._color.theme_color = mso_theme_color_idx

    @property
    def type(self):
        """
        Read-only. A value from :ref:`MsoColorType`, either RGB or SCHEME,
        corresponding to the way this color is defined, or None if no color
        is defined at the level of this font.
        """
        return self._color.color_type

    def _validate_brightness_value(self, value):
        if value < -1.0 or value > 1.0:
            raise ValueError("brightness must be number in range -1.0 to 1.0")
        if isinstance(self._color, _NoneColor):
            msg = (
                "can't set brightness when color.type is None. Set color.rgb"
                " or .theme_color first."
            )
            raise ValueError(msg)


class _Color(object):
    """
    Object factory for color object of the appropriate type, also the base
    class for all color type classes such as SRgbColor.
    """

    def __new__(cls, xClr):
        color_cls = {
            type(None): _NoneColor,
            CT_HslColor: _HslColor,
            CT_PresetColor: _PrstColor,
            CT_SchemeColor: _SchemeColor,
            CT_ScRgbColor: _ScRgbColor,
            CT_SRgbColor: _SRgbColor,
            CT_SystemColor: _SysColor,
        }[type(xClr)]
        return super(_Color, cls).__new__(color_cls)

    def __init__(self, xClr):
        super(_Color, self).__init__()
        self._xClr = xClr

    @property
    def brightness(self):
        lumMod, lumOff = self._xClr.lumMod, self._xClr.lumOff
        # a tint is lighter, a shade is darker
        # only tints have lumOff child
        if lumOff is not None:
            brightness = lumOff.val
            return brightness
        # which leaves shades, if lumMod is present
        if lumMod is not None:
            brightness = lumMod.val - 1.0
            return brightness
        # there's no brightness adjustment if no lum{Mod|Off} elements
        return 0

    @brightness.setter
    def brightness(self, value):
        if value > 0:
            self._tint(value)
        elif value < 0:
            self._shade(value)
        else:
            self._xClr.clear_lum()

    @property
    def color_type(self):  # pragma: no cover
        tmpl = ".color_type property must be implemented on %s"
        raise NotImplementedError(tmpl % self.__class__.__name__)

    @property
    def rgb(self):
        """
        Raises TypeError on access unless overridden by subclass.
        """
        tmpl = "no .rgb property on color type '%s'"
        raise AttributeError(tmpl % self.__class__.__name__)

    @property
    def theme_color(self):
        """
        Raises TypeError on access unless overridden by subclass.
        """
        return MSO_THEME_COLOR.NOT_THEME_COLOR

    def _shade(self, value):
        lumMod_val = 1.0 - abs(value)
        color_elm = self._xClr.clear_lum()
        color_elm.add_lumMod(lumMod_val)

    def _tint(self, value):
        lumOff_val = value
        lumMod_val = 1.0 - lumOff_val
        color_elm = self._xClr.clear_lum()
        color_elm.add_lumMod(lumMod_val)
        color_elm.add_lumOff(lumOff_val)


class _HslColor(_Color):
    @property
    def color_type(self):
        return MSO_COLOR_TYPE.HSL


class _NoneColor(_Color):
    @property
    def color_type(self):
        return None

    @property
    def theme_color(self):
        """
        Raise TypeError on attempt to access .theme_color when no color
        choice is present.
        """
        tmpl = "no .theme_color property on color type '%s'"
        raise AttributeError(tmpl % self.__class__.__name__)


class _PrstColor(_Color):
    @property
    def color_type(self):
        return MSO_COLOR_TYPE.PRESET


class _SchemeColor(_Color):
    def __init__(self, schemeClr):
        super(_SchemeColor, self).__init__(schemeClr)
        self._schemeClr = schemeClr

    @property
    def color_type(self):
        return MSO_COLOR_TYPE.SCHEME

    @property
    def theme_color(self):
        """
        Theme color value of this color, one of those defined in the
        MSO_THEME_COLOR enumeration, e.g. MSO_THEME_COLOR.ACCENT_1. None if
        no theme color is explicitly defined for this font. Setting this to a
        value in MSO_THEME_COLOR causes the color's type to change to
        ``MSO_COLOR_TYPE.SCHEME``.
        """
        return self._schemeClr.val

    @theme_color.setter
    def theme_color(self, mso_theme_color_idx):
        self._schemeClr.val = mso_theme_color_idx


class _ScRgbColor(_Color):
    @property
    def color_type(self):
        return MSO_COLOR_TYPE.SCRGB


class _SRgbColor(_Color):
    def __init__(self, srgbClr):
        super(_SRgbColor, self).__init__(srgbClr)
        self._srgbClr = srgbClr

    @property
    def color_type(self):
        return MSO_COLOR_TYPE.RGB

    @property
    def rgb(self):
        """
        |RGBColor| value of this color, corresponding to the value in the
        required ``val`` attribute of the ``<a:srgbColr>`` element.
        """
        return RGBColor.from_string(self._srgbClr.val)

    @rgb.setter
    def rgb(self, rgb):
        self._srgbClr.val = str(rgb)


class _SysColor(_Color):
    @property
    def color_type(self):
        return MSO_COLOR_TYPE.SYSTEM


class RGBColor(tuple):
    """
    Immutable value object defining a particular RGB color.
    """

    def __new__(cls, r, g, b):
        msg = "RGBColor() takes three integer values 0-255"
        for val in (r, g, b):
            if not isinstance(val, int) or val < 0 or val > 255:
                raise ValueError(msg)
        return super(RGBColor, cls).__new__(cls, (r, g, b))

    def __str__(self):
        """
        Return a hex string rgb value, like '3C2F80'
        """
        return "%02X%02X%02X" % self

    @classmethod
    def from_string(cls, rgb_hex_str):
        """
        Return a new instance from an RGB color hex string like ``'3C2F80'``.
        """
        r = int(rgb_hex_str[:2], 16)
        g = int(rgb_hex_str[2:4], 16)
        b = int(rgb_hex_str[4:], 16)
        return cls(r, g, b)



# encoding: utf-8

"""Test suite for pptx.chart.chart module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.chart.axis import CategoryAxis, DateAxis, ValueAxis
from pptx.chart.chart import Chart, ChartTitle, Legend, _Plots
from pptx.chart.data import ChartData
from pptx.chart.plot import _BasePlot
from pptx.chart.series import SeriesCollection
from pptx.chart.xmlwriter import _BaseSeriesXmlRewriter
from pptx.dml.chtfmt import ChartFormat
from pptx.enum.chart import XL_CHART_TYPE
from pptx.parts.chart import ChartWorkbook
from pptx.text.text import Font

from ..unitutil.cxml import element, xml
from ..unitutil.mock import (
    call,
    class_mock,
    function_mock,
    instance_mock,
    property_mock,
)


class DescribeChart(object):
    def it_provides_access_to_its_font(self, font_fixture, Font_, font_):
        chartSpace, expected_xml = font_fixture
        Font_.return_value = font_
        chart = Chart(chartSpace, None)

        font = chart.font

        assert chartSpace.xml == expected_xml
        Font_.assert_called_once_with(
            chartSpace.xpath("./c:txPr/a:p/a:pPr/a:defRPr")[0]
        )
        assert font is font_

    def it_knows_whether_it_has_a_title(self, has_title_get_fixture):
        chart, expected_value = has_title_get_fixture
        assert chart.has_title is expected_value

    def it_can_change_whether_it_has_a_title(self, has_title_set_fixture):
        chart, new_value, expected_xml = has_title_set_fixture
        chart.has_title = new_value
        assert chart._chartSpace.chart.xml == expected_xml

    def it_provides_access_to_the_chart_title(self, title_fixture):
        chart, expected_xml, ChartTitle_, chart_title_ = title_fixture

        chart_title = chart.chart_title

        assert chart.element.xpath("c:chart/c:title")[0].xml == expected_xml
        ChartTitle_.assert_called_once_with(chart.element.chart.title)
        assert chart_title is chart_title_

    def it_provides_access_to_the_category_axis(self, category_axis_fixture):
        chart, category_axis_, AxisCls_, xAx = category_axis_fixture
        category_axis = chart.category_axis
        AxisCls_.assert_called_once_with(xAx)
        assert category_axis is category_axis_

    def it_raises_when_no_category_axis(self, cat_ax_raise_fixture):
        chart = cat_ax_raise_fixture
        with pytest.raises(ValueError):
            chart.category_axis

    def it_provides_access_to_the_value_axis(self, val_ax_fixture):
        chart, ValueAxis_, valAx, value_axis_ = val_ax_fixture
        value_axis = chart.value_axis
        ValueAxis_.assert_called_once_with(valAx)
        assert value_axis is value_axis_

    def it_raises_when_no_value_axis(self, val_ax_raise_fixture):
        chart = val_ax_raise_fixture
        with pytest.raises(ValueError):
            chart.value_axis

    def it_provides_access_to_its_series(self, series_fixture):
        chart, SeriesCollection_, plotArea, series_ = series_fixture
        series = chart.series
        SeriesCollection_.assert_called_once_with(plotArea)
        assert series is series_

    def it_provides_access_to_its_plots(self, plots_fixture):
        chart, plots_, _Plots_, plotArea = plots_fixture
        plots = chart.plots
        _Plots_.assert_called_once_with(plotArea, chart)
        assert plots is plots_

    def it_knows_whether_it_has_a_legend(self, has_legend_get_fixture):
        chart, expected_value = has_legend_get_fixture
        assert chart.has_legend == expected_value

    def it_can_change_whether_it_has_a_legend(self, has_legend_set_fixture):
        chart, new_value, expected_xml = has_legend_set_fixture
        chart.has_legend = new_value
        assert chart._chartSpace.xml == expected_xml

    def it_provides_access_to_its_legend(self, legend_fixture):
        chart, Legend_, expected_calls, expected_value = legend_fixture
        legend = chart.legend
        assert Legend_.call_args_list == expected_calls
        assert legend is expected_value

    def it_knows_its_chart_type(self, chart_type_fixture):
        chart, PlotTypeInspector_, plot_, chart_type = chart_type_fixture
        _chart_type = chart.chart_type
        PlotTypeInspector_.chart_type.assert_called_once_with(plot_)
        assert _chart_type is chart_type

    def it_knows_its_style(self, style_get_fixture):
        chart, expected_value = style_get_fixture
        assert chart.chart_style == expected_value

    def it_can_change_its_style(self, style_set_fixture):
        chart, new_value, expected_xml = style_set_fixture
        chart.chart_style = new_value
        assert chart._chartSpace.xml == expected_xml

    def it_can_replace_the_chart_data(self, replace_fixture):
        (
            chart,
            chart_data_,
            SeriesXmlRewriterFactory_,
            chart_type,
            rewriter_,
            chartSpace,
            workbook_,
            xlsx_blob,
        ) = replace_fixture

        chart.replace_data(chart_data_)

        SeriesXmlRewriterFactory_.assert_called_once_with(chart_type, chart_data_)
        rewriter_.replace_series_data.assert_called_once_with(chartSpace)
        workbook_.update_from_xlsx_blob.assert_called_once_with(xlsx_blob)

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=["c:catAx", "c:dateAx", "c:valAx"])
    def category_axis_fixture(self, request, CategoryAxis_, DateAxis_, ValueAxis_):
        ax_tag = request.param
        chartSpace_cxml = "c:chartSpace/c:chart/c:plotArea/%s" % ax_tag
        chartSpace = element(chartSpace_cxml)
        chart = Chart(chartSpace, None)
        AxisCls_ = {
            "c:catAx": CategoryAxis_,
            "c:dateAx": DateAxis_,
            "c:valAx": ValueAxis_,
        }[ax_tag]
        axis_ = AxisCls_.return_value
        xAx = chartSpace.xpath(".//%s" % ax_tag)[0]
        return chart, axis_, AxisCls_, xAx

    @pytest.fixture
    def cat_ax_raise_fixture(self):
        chart = Chart(element("c:chartSpace/c:chart/c:plotArea"), None)
        return chart

    @pytest.fixture
    def chart_type_fixture(self, PlotTypeInspector_, plot_):
        chart = Chart(None, None)
        chart._plots = [plot_]
        chart_type = XL_CHART_TYPE.PIE
        PlotTypeInspector_.chart_type.return_value = chart_type
        return chart, PlotTypeInspector_, plot_, chart_type

    @pytest.fixture(
        params=[
            (
                "c:chartSpace{a:b=c}",
                "c:chartSpace{a:b=c}/c:txPr/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr"
                ")",
            ),
            ("c:chartSpace/c:txPr/a:p", "c:chartSpace/c:txPr/a:p/a:pPr/a:defRPr"),
            (
                "c:chartSpace/c:txPr/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr)",
                "c:chartSpace/c:txPr/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr)",
            ),
        ]
    )
    def font_fixture(self, request):
        chartSpace_cxml, expected_cxml = request.param
        chartSpace = element(chartSpace_cxml)
        expected_xml = xml(expected_cxml)
        return chartSpace, expected_xml

    @pytest.fixture(
        params=[
            ("c:chartSpace/c:chart", False),
            ("c:chartSpace/c:chart/c:legend", True),
        ]
    )
    def has_legend_get_fixture(self, request):
        chartSpace_cxml, expected_value = request.param
        chart = Chart(element(chartSpace_cxml), None)
        return chart, expected_value

    @pytest.fixture(
        params=[("c:chartSpace/c:chart", True, "c:chartSpace/c:chart/c:legend")]
    )
    def has_legend_set_fixture(self, request):
        chartSpace_cxml, new_value, expected_chartSpace_cxml = request.param
        chart = Chart(element(chartSpace_cxml), None)
        expected_xml = xml(expected_chartSpace_cxml)
        return chart, new_value, expected_xml

    @pytest.fixture(
        params=[("c:chartSpace/c:chart", False), ("c:chartSpace/c:chart/c:title", True)]
    )
    def has_title_get_fixture(self, request):
        chartSpace_cxml, expected_value = request.param
        chart = Chart(element(chartSpace_cxml), None)
        return chart, expected_value

    @pytest.fixture(
        params=[
            ("c:chart", True, "c:chart/c:title/(c:layout,c:overlay{val=0})"),
            ("c:chart/c:title", True, "c:chart/c:title"),
            ("c:chart/c:title", False, "c:chart/c:autoTitleDeleted{val=1}"),
            ("c:chart", False, "c:chart/c:autoTitleDeleted{val=1}"),
        ]
    )
    def has_title_set_fixture(self, request):
        chart_cxml, new_value, expected_cxml = request.param
        chart = Chart(element("c:chartSpace/%s" % chart_cxml), None)
        expected_xml = xml(expected_cxml)
        return chart, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("c:chartSpace/c:chart", False),
            ("c:chartSpace/c:chart/c:legend", True),
        ]
    )
    def legend_fixture(self, request, Legend_, legend_):
        chartSpace_cxml, has_legend = request.param
        chartSpace = element(chartSpace_cxml)
        chart = Chart(chartSpace, None)
        expected_value, expected_calls = None, []
        if has_legend:
            expected_value = legend_
            legend_elm = chartSpace.chart.legend
            expected_calls.append(call(legend_elm))
        return chart, Legend_, expected_calls, expected_value

    @pytest.fixture
    def plots_fixture(self, _Plots_, plots_):
        chartSpace = element("c:chartSpace/c:chart/c:plotArea")
        plotArea = chartSpace.xpath("./c:chart/c:plotArea")[0]
        chart = Chart(chartSpace, None)
        return chart, plots_, _Plots_, plotArea

    @pytest.fixture
    def replace_fixture(
        self,
        chart_data_,
        SeriesXmlRewriterFactory_,
        series_rewriter_,
        workbook_,
        workbook_prop_,
    ):
        chartSpace = element("c:chartSpace/c:chart/c:plotArea/c:pieChart")
        chart = Chart(chartSpace, None)
        chart_type = XL_CHART_TYPE.PIE
        xlsx_blob = "fooblob"
        chart_data_.xlsx_blob = xlsx_blob
        return (
            chart,
            chart_data_,
            SeriesXmlRewriterFactory_,
            chart_type,
            series_rewriter_,
            chartSpace,
            workbook_,
            xlsx_blob,
        )

    @pytest.fixture
    def series_fixture(self, SeriesCollection_, series_collection_):
        chartSpace = element("c:chartSpace/c:chart/c:plotArea")
        plotArea = chartSpace.xpath(".//c:plotArea")[0]
        chart = Chart(chartSpace, None)
        return chart, SeriesCollection_, plotArea, series_collection_

    @pytest.fixture(
        params=[("c:chartSpace/c:style{val=42}", 42), ("c:chartSpace", None)]
    )
    def style_get_fixture(self, request):
        chartSpace_cxml, expected_value = request.param
        chart = Chart(element(chartSpace_cxml), None)
        return chart, expected_value

    @pytest.fixture(
        params=[
            ("c:chartSpace", 4, "c:chartSpace/c:style{val=4}"),
            ("c:chartSpace", None, "c:chartSpace"),
            ("c:chartSpace/c:style{val=4}", 2, "c:chartSpace/c:style{val=2}"),
            ("c:chartSpace/c:style{val=4}", None, "c:chartSpace"),
        ]
    )
    def style_set_fixture(self, request):
        chartSpace_cxml, new_value, expected_chartSpace_cxml = request.param
        chart = Chart(element(chartSpace_cxml), None)
        expected_xml = xml(expected_chartSpace_cxml)
        return chart, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("c:chartSpace/c:chart", "c:title/(c:layout,c:overlay{val=0})"),
            ("c:chartSpace/c:chart/c:title/c:layout", "c:title/c:layout"),
        ]
    )
    def title_fixture(self, request, ChartTitle_, chart_title_):
        chartSpace_cxml, expected_cxml = request.param
        chart = Chart(element(chartSpace_cxml), None)
        expected_xml = xml(expected_cxml)
        return chart, expected_xml, ChartTitle_, chart_title_

    @pytest.fixture(
        params=[
            ("c:chartSpace/c:chart/c:plotArea/(c:catAx,c:valAx)", 0),
            ("c:chartSpace/c:chart/c:plotArea/(c:valAx,c:valAx)", 1),
        ]
    )
    def val_ax_fixture(self, request, ValueAxis_, value_axis_):
        chartSpace_xml, idx = request.param
        chartSpace = element(chartSpace_xml)
        chart = Chart(chartSpace, None)
        valAx = chartSpace.xpath(".//c:valAx")[idx]
        return chart, ValueAxis_, valAx, value_axis_

    @pytest.fixture
    def val_ax_raise_fixture(self):
        chart = Chart(element("c:chartSpace/c:chart/c:plotArea"), None)
        return chart

    # fixture components ---------------------------------------------

    @pytest.fixture
    def CategoryAxis_(self, request, category_axis_):
        return class_mock(
            request, "pptx.chart.chart.CategoryAxis", return_value=category_axis_
        )

    @pytest.fixture
    def category_axis_(self, request):
        return instance_mock(request, CategoryAxis)

    @pytest.fixture
    def chart_data_(self, request):
        return instance_mock(request, ChartData)

    @pytest.fixture
    def ChartTitle_(self, request, chart_title_):
        return class_mock(
            request, "pptx.chart.chart.ChartTitle", return_value=chart_title_
        )

    @pytest.fixture
    def chart_title_(self, request):
        return instance_mock(request, ChartTitle)

    @pytest.fixture
    def DateAxis_(self, request, date_axis_):
        return class_mock(request, "pptx.chart.chart.DateAxis", return_value=date_axis_)

    @pytest.fixture
    def date_axis_(self, request):
        return instance_mock(request, DateAxis)

    @pytest.fixture
    def Font_(self, request):
        return class_mock(request, "pptx.chart.chart.Font")

    @pytest.fixture
    def font_(self, request):
        return instance_mock(request, Font)

    @pytest.fixture
    def Legend_(self, request, legend_):
        return class_mock(request, "pptx.chart.chart.Legend", return_value=legend_)

    @pytest.fixture
    def legend_(self, request):
        return instance_mock(request, Legend)

    @pytest.fixture
    def PlotTypeInspector_(self, request):
        return class_mock(request, "pptx.chart.chart.PlotTypeInspector")

    @pytest.fixture
    def _Plots_(self, request, plots_):
        return class_mock(request, "pptx.chart.chart._Plots", return_value=plots_)

    @pytest.fixture
    def plot_(self, request):
        return instance_mock(request, _BasePlot)

    @pytest.fixture
    def plots_(self, request):
        return instance_mock(request, _Plots)

    @pytest.fixture
    def SeriesCollection_(self, request, series_collection_):
        return class_mock(
            request,
            "pptx.chart.chart.SeriesCollection",
            return_value=series_collection_,
        )

    @pytest.fixture
    def SeriesXmlRewriterFactory_(self, request, series_rewriter_):
        return function_mock(
            request,
            "pptx.chart.chart.SeriesXmlRewriterFactory",
            return_value=series_rewriter_,
            autospec=True,
        )

    @pytest.fixture
    def series_collection_(self, request):
        return instance_mock(request, SeriesCollection)

    @pytest.fixture
    def series_rewriter_(self, request):
        return instance_mock(request, _BaseSeriesXmlRewriter)

    @pytest.fixture
    def ValueAxis_(self, request, value_axis_):
        return class_mock(
            request, "pptx.chart.chart.ValueAxis", return_value=value_axis_
        )

    @pytest.fixture
    def value_axis_(self, request):
        return instance_mock(request, ValueAxis)

    @pytest.fixture
    def workbook_(self, request):
        return instance_mock(request, ChartWorkbook)

    @pytest.fixture
    def workbook_prop_(self, request, workbook_):
        return property_mock(request, Chart, "_workbook", return_value=workbook_)


class DescribeChartTitle(object):
    def it_provides_access_to_its_format(self, format_fixture):
        chart_title, ChartFormat_, format_ = format_fixture
        format = chart_title.format
        ChartFormat_.assert_called_once_with(chart_title.element)
        assert format is format_

    def it_knows_whether_it_has_a_text_frame(self, has_tf_get_fixture):
        chart_title, expected_value = has_tf_get_fixture
        value = chart_title.has_text_frame
        assert value is expected_value

    def it_can_change_whether_it_has_a_text_frame(self, has_tf_set_fixture):
        chart_title, value, expected_xml = has_tf_set_fixture
        chart_title.has_text_frame = value
        assert chart_title._element.xml == expected_xml

    def it_provides_access_to_its_text_frame(self, text_frame_fixture):
        chart_title, TextFrame_, text_frame_ = text_frame_fixture
        text_frame = chart_title.text_frame
        TextFrame_.assert_called_once_with(chart_title._element.tx.rich, chart_title)
        assert text_frame is text_frame_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def format_fixture(self, request, ChartFormat_, format_):
        chart_title = ChartTitle(element("c:title"))
        return chart_title, ChartFormat_, format_

    @pytest.fixture(
        params=[
            ("c:title", False),
            ("c:title/c:tx", False),
            ("c:title/c:tx/c:strRef", False),
            ("c:title/c:tx/c:rich", True),
        ]
    )
    def has_tf_get_fixture(self, request):
        title_cxml, expected_value = request.param
        chart_title = ChartTitle(element(title_cxml))
        return chart_title, expected_value

    @pytest.fixture(
        params=[
            (
                "c:title{a:b=c}",
                True,
                "c:title{a:b=c}/c:tx/c:rich/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr"
                ")",
            ),
            (
                "c:title{a:b=c}/c:tx",
                True,
                "c:title{a:b=c}/c:tx/c:rich/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr"
                ")",
            ),
            (
                "c:title{a:b=c}/c:tx/c:strRef",
                True,
                "c:title{a:b=c}/c:tx/c:rich/(a:bodyPr,a:lstStyle,a:p/a:pPr/a:defRPr"
                ")",
            ),
            ("c:title/c:tx/c:rich", True, "c:title/c:tx/c:rich"),
            ("c:title", False, "c:title"),
            ("c:title/c:tx", False, "c:title"),
            ("c:title/c:tx/c:rich", False, "c:title"),
            ("c:title/c:tx/c:strRef", False, "c:title"),
        ]
    )
    def has_tf_set_fixture(self, request):
        title_cxml, value, expected_cxml = request.param
        chart_title = ChartTitle(element(title_cxml))
        expected_xml = xml(expected_cxml)
        return chart_title, value, expected_xml

    @pytest.fixture
    def text_frame_fixture(self, request, TextFrame_):
        chart_title = ChartTitle(element("c:title"))
        text_frame_ = TextFrame_.return_value
        return chart_title, TextFrame_, text_frame_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def ChartFormat_(self, request, format_):
        return class_mock(request, "pptx.chart.chart.ChartFormat", return_value=format_)

    @pytest.fixture
    def format_(self, request):
        return instance_mock(request, ChartFormat)

    @pytest.fixture
    def TextFrame_(self, request):
        return class_mock(request, "pptx.chart.chart.TextFrame")


class Describe_Plots(object):
    def it_supports_indexed_access(self, getitem_fixture):
        plots, idx, PlotFactory_, plot_elm, chart_, plot_ = getitem_fixture
        plot = plots[idx]
        PlotFactory_.assert_called_once_with(plot_elm, chart_)
        assert plot is plot_

    def it_supports_len(self, len_fixture):
        plots, expected_len = len_fixture
        assert len(plots) == expected_len

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:plotArea/c:barChart", 0),
            ("c:plotArea/(c:radarChart,c:barChart)", 1),
        ]
    )
    def getitem_fixture(self, request, PlotFactory_, chart_, plot_):
        plotArea_cxml, idx = request.param
        plotArea = element(plotArea_cxml)
        plot_elm = plotArea[idx]
        plots = _Plots(plotArea, chart_)
        return plots, idx, PlotFactory_, plot_elm, chart_, plot_

    @pytest.fixture(
        params=[
            ("c:plotArea", 0),
            ("c:plotArea/c:barChart", 1),
            ("c:plotArea/(c:barChart,c:lineChart)", 2),
        ]
    )
    def len_fixture(self, request):
        plotArea_cxml, expected_len = request.param
        plots = _Plots(element(plotArea_cxml), None)
        return plots, expected_len

    # fixture components ---------------------------------------------

    @pytest.fixture
    def chart_(self, request):
        return instance_mock(request, Chart)

    @pytest.fixture
    def PlotFactory_(self, request, plot_):
        return function_mock(
            request, "pptx.chart.chart.PlotFactory", return_value=plot_
        )

    @pytest.fixture
    def plot_(self, request):
        return instance_mock(request, _BasePlot)
